# sage_setup: distribution = sagemath-categories
r"""
Ring `\ZZ/n\ZZ` of integers modulo `n`

EXAMPLES::

    sage: R = Integers(97)
    sage: a = R(5)
    sage: a**100000000000000000000000000000000000000000000000000000000000000
    61

This example illustrates the relation between
`\ZZ/p\ZZ` and `\GF{p}`. In
particular, there is a canonical map to `\GF{p}`, but not in
the other direction.

::

    sage: r = Integers(7)
    sage: s = GF(7)
    sage: r.has_coerce_map_from(s)
    False
    sage: s.has_coerce_map_from(r)
    True
    sage: s(1) + r(1)
    2
    sage: parent(s(1) + r(1))
    Finite Field of size 7
    sage: parent(r(1) + s(1))
    Finite Field of size 7

We list the elements of `\ZZ/3\ZZ`::

    sage: R = Integers(3)
    sage: list(R)
    [0, 1, 2]

AUTHORS:

- William Stein (initial code)

- David Joyner (2005-12-22): most examples

- Robert Bradshaw (2006-08-24): convert to SageX (Cython)

- William Stein (2007-04-29): square_roots_of_one

- Simon King (2011-04-21): allow to prescribe a category

- Simon King (2013-09): Only allow to prescribe the category of fields

- Kyle Hofmann (2024-02): New implementation of root-finding
"""

# ****************************************************************************
#       Copyright (C) 2005 William Stein <wstein@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************


import sage.misc.prandom as random

from sage.arith.misc import factor
from sage.arith.misc import primitive_root
from sage.arith.misc import CRT_basis
from sage.rings.ring import Field, CommutativeRing
from sage.misc.mrange import cartesian_product_iterator
import sage.rings.ring as ring
import sage.rings.abc
from sage.rings.finite_rings import integer_mod
import sage.rings.integer as integer
import sage.rings.integer_ring as integer_ring
import sage.rings.quotient_ring as quotient_ring

try:
    from sage.libs.pari.all import pari, PariError
except ImportError:
    class PariError(Exception):
        pass

from sage.misc.cachefunc import cached_method

from sage.structure.factory import UniqueFactory
from sage.structure.richcmp import richcmp, richcmp_method

from sage.interfaces.abc import GapElement


class IntegerModFactory(UniqueFactory):
    r"""
    Return the quotient ring `\ZZ / n\ZZ`.

    INPUT:

    - ``order`` -- integer (default: 0); positive or negative
    - ``is_field`` -- boolean (default: ``False``); assert that the order is
      prime and hence the quotient ring belongs to the category of fields
    - ``category`` -- (optional) the category that the quotient ring belongs to

    .. NOTE::

        The optional argument ``is_field`` is not part of the cache key.
        Hence, this factory will create precisely one instance of `\ZZ /
        n\ZZ`.  However, if ``is_field`` is true, then a previously created
        instance of the quotient ring will be updated to be in the category of
        fields.

        **Use with care!** Erroneously putting `\ZZ / n\ZZ` into the category
        of fields may have consequences that can compromise a whole Sage
        session, so that a restart will be needed.

    EXAMPLES::

        sage: IntegerModRing(15)
        Ring of integers modulo 15
        sage: IntegerModRing(7)
        Ring of integers modulo 7
        sage: IntegerModRing(-100)
        Ring of integers modulo 100

    Note that you can also use ``Integers``, which is a
    synonym for ``IntegerModRing``.

    ::

        sage: Integers(18)
        Ring of integers modulo 18
        sage: Integers() is Integers(0) is ZZ
        True

    .. NOTE::

        Testing whether a quotient ring `\ZZ / n\ZZ` is a field can of
        course be very costly. By default, it is not tested whether `n`
        is prime or not, in contrast to
        :func:`~sage.rings.finite_rings.finite_field_constructor.GF`. If the user
        is sure that the modulus is prime and wants to avoid a primality
        test, (s)he can provide ``category=Fields()`` when constructing
        the quotient ring, and then the result will behave like a field.
        If the category is not provided during initialisation, and it is
        found out later that the ring is in fact a field, then the category
        will be changed at runtime, having the same effect as providing
        ``Fields()`` during initialisation.

    EXAMPLES::

        sage: R = IntegerModRing(5)
        sage: R.category()
        Join of Category of finite commutative rings
            and Category of subquotients of monoids
            and Category of quotients of semigroups
            and Category of finite enumerated sets
        sage: R in Fields()
        True
        sage: R.category()
        Join of Category of finite enumerated fields
            and Category of subquotients of monoids
            and Category of quotients of semigroups
        sage: S = IntegerModRing(5, is_field=True)
        sage: S is R
        True

    .. WARNING::

        If the optional argument ``is_field`` was used by mistake, there is
        currently no way to revert its impact, even though
        :meth:`IntegerModRing_generic.is_field` with the optional argument
        ``proof=True`` would return the correct answer.  So, prescribe
        ``is_field=True`` only if you know what your are doing!

    EXAMPLES::

        sage: R = IntegerModRing(33, is_field=True)
        sage: R in Fields()
        True
        sage: R.is_field()
        True

    If the optional argument `proof=True` is provided, primality is tested and
    the mistaken category assignment is reported::

        sage: R.is_field(proof=True)
        Traceback (most recent call last):
        ...
        ValueError: THIS SAGE SESSION MIGHT BE SERIOUSLY COMPROMISED!
        The order 33 is not prime, but this ring has been put
        into the category of fields. This may already have consequences
        in other parts of Sage. Either it was a mistake of the user,
        or a probabilistic primality test has failed.
        In the latter case, please inform the developers.

    However, the mistaken assignment is not automatically corrected::

        sage: R in Fields()
        True

    To avoid side-effects of this test on other tests, we clear the cache of
    the ring factory::

        sage: IntegerModRing._cache.clear()
    """
    def get_object(self, version, key, extra_args):
        out = super().get_object(version, key, extra_args)
        category = extra_args.get('category', None)
        if category is not None:
            out._refine_category_(category)
            out._factory_data[3]['category'] = category
        return out

    def create_key_and_extra_args(self, order=0, is_field=False, category=None):
        """
        An integer mod ring is specified uniquely by its order.

        EXAMPLES::

            sage: Zmod.create_key_and_extra_args(7)
            (7, {})
            sage: Zmod.create_key_and_extra_args(7, True)
            (7, {'category': Category of fields})
        """
        if is_field:
            from sage.categories.fields import Fields
            return order, {'category': Fields()}
        return order, {}

    def create_object(self, version, order, **kwds):
        """
        EXAMPLES::

            sage: R = Integers(10)
            sage: TestSuite(R).run() # indirect doctest
        """
        if isinstance(order, tuple):
            # this is for unpickling old data
            order, category = order
            kwds.setdefault('category', category)
        if order < 0:
            order = -order
        if order == 0:
            return integer_ring.IntegerRing(**kwds)
        else:
            return IntegerModRing_generic(order, **kwds)


Zmod = Integers = IntegerModRing = IntegerModFactory("IntegerModRing")


from sage.categories.commutative_rings import CommutativeRings
from sage.categories.finite_enumerated_sets import FiniteEnumeratedSets
from sage.categories.category import JoinCategory
default_category = JoinCategory((CommutativeRings(), FiniteEnumeratedSets()))
ZZ = integer_ring.IntegerRing()


def _unit_gens_primepowercase(p, r):
    r"""
    Return a list of generators for `(\ZZ/p^r\ZZ)^*` and their orders.

    EXAMPLES::

        sage: from sage.rings.finite_rings.integer_mod_ring import _unit_gens_primepowercase
        sage: _unit_gens_primepowercase(2, 3)
        [(7, 2), (5, 2)]
        sage: _unit_gens_primepowercase(17, 1)                                          # needs sage.libs.pari
        [(3, 16)]
        sage: _unit_gens_primepowercase(3, 3)                                           # needs sage.libs.pari
        [(2, 18)]
    """
    pr = p**r
    if p == 2:
        if r == 1:
            return []
        if r == 2:
            return [(integer_mod.Mod(3, 4), integer.Integer(2))]
        return [(integer_mod.Mod(-1, pr), integer.Integer(2)),
                (integer_mod.Mod(5, pr), integer.Integer(2**(r - 2)))]

    # odd prime
    return [(integer_mod.Mod(primitive_root(pr, check=False), pr),
             integer.Integer(p**(r - 1) * (p - 1)))]


@richcmp_method
class IntegerModRing_generic(quotient_ring.QuotientRing_generic, sage.rings.abc.IntegerModRing):
    """
    The ring of integers modulo `N`.

    INPUT:

    - ``order`` -- integer

    - ``category`` -- a subcategory of ``CommutativeRings()`` (the default)

    OUTPUT: the ring of integers modulo `N`

    EXAMPLES:

    First we compute with integers modulo `29`.

    ::

        sage: FF = IntegerModRing(29)
        sage: FF
        Ring of integers modulo 29
        sage: FF.category()
        Join of Category of finite commutative rings
            and Category of subquotients of monoids
            and Category of quotients of semigroups
            and Category of finite enumerated sets
        sage: FF.is_field()
        True
        sage: FF.characteristic()
        29
        sage: FF.order()
        29

        sage: # needs sage.groups
        sage: gens = FF.unit_gens()
        sage: a = gens[0]
        sage: a
        2
        sage: a.is_square()
        False
        sage: def pow(i): return a**i
        sage: [pow(i) for i in range(16)]
        [1, 2, 4, 8, 16, 3, 6, 12, 24, 19, 9, 18, 7, 14, 28, 27]
        sage: TestSuite(FF).run()

    We have seen above that an integer mod ring is, by default, not
    initialised as an object in the category of fields. However, one
    can force it to be. Moreover, testing containment in the category
    of fields my re-initialise the category of the integer mod ring::

        sage: F19 = IntegerModRing(19, is_field=True)
        sage: F19.category().is_subcategory(Fields())
        True
        sage: F23 = IntegerModRing(23)
        sage: F23.category().is_subcategory(Fields())
        False
        sage: F23 in Fields()
        True
        sage: F23.category().is_subcategory(Fields())
        True
        sage: TestSuite(F19).run()
        sage: TestSuite(F23).run()

    By :issue:`15229`, there is a unique instance of the
    integral quotient ring of a given order. Using the
    :func:`IntegerModRing` factory twice, and using
    ``is_field=True`` the second time, will update the
    category of the unique instance::

        sage: F31a = IntegerModRing(31)
        sage: F31a.category().is_subcategory(Fields())
        False
        sage: F31b = IntegerModRing(31, is_field=True)
        sage: F31a is F31b
        True
        sage: F31a.category().is_subcategory(Fields())
        True

    Next we compute with the integers modulo `16`.

    ::

        sage: Z16 = IntegerModRing(16)
        sage: Z16.category()
        Join of Category of finite commutative rings
            and Category of subquotients of monoids
            and Category of quotients of semigroups
            and Category of finite enumerated sets
        sage: Z16.is_field()
        False
        sage: Z16.order()
        16
        sage: Z16.characteristic()
        16

        sage: # needs sage.groups
        sage: gens = Z16.unit_gens()
        sage: gens
        (15, 5)
        sage: a = gens[0]
        sage: b = gens[1]
        sage: def powa(i): return a**i
        sage: def powb(i): return b**i
        sage: gp_exp = FF.unit_group_exponent()
        sage: gp_exp
        28
        sage: [powa(i) for i in range(15)]
        [1, 15, 1, 15, 1, 15, 1, 15, 1, 15, 1, 15, 1, 15, 1]
        sage: [powb(i) for i in range(15)]
        [1, 5, 9, 13, 1, 5, 9, 13, 1, 5, 9, 13, 1, 5, 9]
        sage: a.multiplicative_order()
        2
        sage: b.multiplicative_order()
        4

        sage: TestSuite(Z16).run()

    Saving and loading::

        sage: R = Integers(100000)
        sage: TestSuite(R).run()  # long time (17s on sage.math, 2011)

    Testing ideals and quotients::

        sage: Z10 = Integers(10)
        sage: I = Z10.principal_ideal(0)
        sage: Z10.quotient(I) == Z10
        True
        sage: I = Z10.principal_ideal(2)
        sage: Z10.quotient(I) == Z10
        False
        sage: I.is_prime()
        True

    ::

        sage: R = IntegerModRing(97)
        sage: a = R(5)
        sage: a**(10^62)
        61
    """
    def __init__(self, order, cache=None, category=None):
        """
        Create with the command ``IntegerModRing(order)``.

        TESTS::

            sage: FF = IntegerModRing(29)
            sage: TestSuite(FF).run()
            sage: F19 = IntegerModRing(19, is_field=True)
            sage: TestSuite(F19).run()
            sage: F23 = IntegerModRing(23)
            sage: F23 in Fields()
            True
            sage: TestSuite(F23).run()
            sage: Z16 = IntegerModRing(16)
            sage: TestSuite(Z16).run()
            sage: R = Integers(100000)
            sage: TestSuite(R).run()  # long time (17s on sage.math, 2011)

            sage: R = IntegerModRing(18)
            sage: R.is_finite()
            True
        """
        order = ZZ(order)
        if order <= 0:
            raise ZeroDivisionError("order must be positive")
        self.__order = order
        self._pyx_order = integer_mod.NativeIntStruct(order)
        global default_category
        if category is None:
            category = default_category
        else:
            # If the category is given, e.g., as Fields(), then we still
            # know that the result will also live in default_category.
            # Hence, we use the join of the default and the given category.
            category = category.join([category, default_category])
        # Give the generator a 'name' to make quotients work.  The
        # name 'x' is used because it's also used for the ring of
        # integers: see the __init__ method for IntegerRing_class in
        # sage/rings/integer_ring.pyx.
        quotient_ring.QuotientRing_generic.__init__(self, ZZ, ZZ.ideal(order),
                                                    names=('x',),
                                                    category=category)
        # We want that the ring is its own base ring.
        self._base = self
        if cache is None:
            cache = order < 500
        if cache:
            self._precompute_table()
        self._zero_element = integer_mod.IntegerMod(self, 0)
        self._one_element = integer_mod.IntegerMod(self, 1)

    def _macaulay2_init_(self, macaulay2=None):
        """
        EXAMPLES::

            sage: macaulay2(Integers(7))  # optional - macaulay2
            ZZ
            --
             7

        ::

            sage: macaulay2(Integers(10)) # optional - macaulay2
            Traceback (most recent call last):
            ...
            TypeError: Error evaluating Macaulay2 code.
            IN:...
            OUT:...error: ZZ/n not implemented yet for composite n...
        """
        return "ZZ/{}".format(self.order())

    def _axiom_init_(self):
        """
        Return a string representation of ``self`` in (Pan)Axiom.

        EXAMPLES::

            sage: Z7 = Integers(7)
            sage: Z7._axiom_init_()
            'IntegerMod(7)'

            sage: axiom(Z7)  #optional - axiom
            IntegerMod 7

            sage: fricas(Z7) #optional - fricas
            IntegerMod(7)
        """
        return 'IntegerMod({})'.format(self.order())

    _fricas_init_ = _axiom_init_

    def krull_dimension(self):
        """
        Return the Krull dimension of ``self``.

        EXAMPLES::

            sage: Integers(18).krull_dimension()
            0
        """
        return integer.Integer(0)

    def is_noetherian(self):
        """
        Check if ``self`` is a Noetherian ring.

        EXAMPLES::

            sage: Integers(8).is_noetherian()
            True
        """
        return True

    def extension(self, poly, name=None, names=None, **kwds):
        """
        Return an algebraic extension of ``self``. See
        :meth:`sage.rings.ring.CommutativeRing.extension()` for more
        information.

        EXAMPLES::

            sage: R.<t> = QQ[]
            sage: Integers(8).extension(t^2 - 3)
            Univariate Quotient Polynomial Ring in t
             over Ring of integers modulo 8 with modulus t^2 + 5
        """
        if self.modulus() == 1:
            return self

        from sage.rings.ring import CommutativeRing
        return CommutativeRing.extension(self, poly, name, names, **kwds)

    @cached_method
    def is_prime_field(self):
        """
        Return ``True`` if the order is prime.

        EXAMPLES::

            sage: Zmod(7).is_prime_field()
            True
            sage: Zmod(8).is_prime_field()
            False
        """
        return self.__order.is_prime()

    def _precompute_table(self):
        """
        Compute a table of elements so that elements are unique.

        EXAMPLES::

            sage: R = Zmod(500); R._precompute_table()
            sage: R(7) + R(13) is R(3) + R(17)
            True
        """
        self._pyx_order.precompute_table(self)

    def list_of_elements_of_multiplicative_group(self):
        """
        Return a list of all invertible elements, as python ints.

        EXAMPLES::

            sage: R = Zmod(12)
            sage: L = R.list_of_elements_of_multiplicative_group(); L
            [1, 5, 7, 11]
            sage: type(L[0])
            <... 'int'>
            sage: Zmod(1).list_of_elements_of_multiplicative_group()
            [0]
        """
        import sage.rings.fast_arith as a
        if self.__order <= 46340:   # todo: don't hard code
            gcd = a.arith_int().gcd_int
        elif self.__order <= 2147483647:   # todo: don't hard code
            gcd = a.arith_llong().gcd_longlong
        else:
            raise NotImplementedError("list_of_elements_of_multiplicative_group() is not implemented for large moduli")
        N = self.__order
        # Don't use N.coprime_integers() here because we want Python ints
        return [i for i in range(N) if gcd(i, N) == 1]

    @cached_method
    def multiplicative_subgroups(self):
        r"""
        Return generators for each subgroup of `(\ZZ/N\ZZ)^*`.

        EXAMPLES::

            sage: # optional - gap_package_polycyclic, needs sage.groups
            sage: Integers(5).multiplicative_subgroups()
            ((2,), (4,), ())
            sage: Integers(15).multiplicative_subgroups()
            ((11, 7), (11, 4), (2,), (11,), (14,), (7,), (4,), ())
            sage: Integers(2).multiplicative_subgroups()
            ((),)
            sage: len(Integers(341).multiplicative_subgroups())
            80

        TESTS::

            sage: IntegerModRing(1).multiplicative_subgroups()                          # needs sage.groups
            ((),)
            sage: IntegerModRing(2).multiplicative_subgroups()                          # needs sage.groups
            ((),)
            sage: IntegerModRing(3).multiplicative_subgroups()  # optional - gap_package_polycyclic, needs sage.groups
            ((2,), ())
        """
        return tuple(tuple(g.value() for g in H.gens())
                     for H in self.unit_group().subgroups())

    def is_integral_domain(self, proof=None):
        """
        Return ``True`` if and only if the order of ``self`` is prime.

        EXAMPLES::

            sage: Integers(389).is_integral_domain()
            True
            sage: Integers(389^2).is_integral_domain()                                  # needs sage.libs.pari
            False

        TESTS:

        Check that :issue:`17453` is fixed::

            sage: R = Zmod(5)
            sage: R in IntegralDomains()
            True
        """
        return self.is_field(proof)

    def is_unique_factorization_domain(self, proof=None):
        """
        Return ``True`` if and only if the order of ``self`` is prime.

        EXAMPLES::

            sage: Integers(389).is_unique_factorization_domain()
            True
            sage: Integers(389^2).is_unique_factorization_domain()                      # needs sage.libs.pari
            False
        """
        return self.is_field(proof)

    @cached_method
    def is_field(self, proof=None):
        r"""
        Return ``True`` precisely if the order is prime.

        INPUT:

        - ``proof`` -- boolean or ``None`` (default). If ``False``, then test
          whether the category of the quotient is a subcategory of
          ``Fields()``, or do a probabilistic primality test. If ``None``, then
          test the category and then do a primality test according to the
          global arithmetic proof settings. If ``True``, do a deterministic
          primality test.

        If it is found (perhaps probabilistically) that the ring is a field,
        then the category of the ring is refined to include the category
        of fields. This may change the Python class of the ring!

        EXAMPLES::

            sage: R = IntegerModRing(18)
            sage: R.is_field()
            False
            sage: FF = IntegerModRing(17)
            sage: FF.is_field()
            True

        By :issue:`15229`, the category of the ring is refined,
        if it is found that the ring is in fact a field::

            sage: R = IntegerModRing(127)
            sage: R.category()
            Join of Category of finite commutative rings
                and Category of subquotients of monoids
                and Category of quotients of semigroups
                and Category of finite enumerated sets
            sage: R.is_field()
            True
            sage: R.category()
            Join of Category of finite enumerated fields
                and Category of subquotients of monoids
                and Category of quotients of semigroups

        It is possible to mistakenly put `\ZZ/n\ZZ` into the category of fields.
        In this case, :meth:`is_field` will return ``True`` without performing a
        primality check. However, if the optional argument ``proof=True`` is
        provided, primality is tested and the mistake is uncovered in a warning
        message::

            sage: R = IntegerModRing(21, is_field=True)
            sage: R.is_field()
            True
            sage: R.is_field(proof=True)
            Traceback (most recent call last):
            ...
            ValueError: THIS SAGE SESSION MIGHT BE SERIOUSLY COMPROMISED!
            The order 21 is not prime, but this ring has been put
            into the category of fields. This may already have consequences
            in other parts of Sage. Either it was a mistake of the user,
            or a probabilistic primality test has failed.
            In the latter case, please inform the developers.

        To avoid side-effects of this test on other tests, we clear the cache
        of the ring factory::

            sage: IntegerModRing._cache.clear()
        """
        from sage.categories.fields import Fields
        if not proof:
            if self.category().is_subcategory(Fields()):
                return True
        is_prime = self.order().is_prime(proof=proof)
        if is_prime:
            self._refine_category_(Fields())
            self._factory_data[3]['category'] = Fields()
        else:
            if self.category().is_subcategory(Fields()):
                raise ValueError("""THIS SAGE SESSION MIGHT BE SERIOUSLY COMPROMISED!
The order {} is not prime, but this ring has been put
into the category of fields. This may already have consequences
in other parts of Sage. Either it was a mistake of the user,
or a probabilistic primality test has failed.
In the latter case, please inform the developers.""".format(self.order()))
        return is_prime

    @cached_method
    def field(self):
        """
        If this ring is a field, return the corresponding field as a finite
        field, which may have extra functionality and structure. Otherwise,
        raise a :exc:`ValueError`.

        EXAMPLES::

            sage: R = Integers(7); R
            Ring of integers modulo 7
            sage: R.field()
            Finite Field of size 7
            sage: R = Integers(9)
            sage: R.field()
            Traceback (most recent call last):
            ...
            ValueError: self must be a field
        """
        try:
            return self.__field
        except AttributeError:
            if not self.is_field():
                raise ValueError("self must be a field")
            from . import finite_field_constructor
            k = finite_field_constructor.FiniteField(self.order())
            self.__field = k
            return k

    def _pseudo_fraction_field(self):
        """
        If ``self`` is composite, we may still want to do division by elements
        of ``self``.

        EXAMPLES::

            sage: Integers(15).fraction_field()
            Traceback (most recent call last):
            ...
            TypeError: self must be an integral domain.
            sage: Integers(15)._pseudo_fraction_field()
            Ring of integers modulo 15
            sage: R.<x> = Integers(15)[]
            sage: (x+5)/2
            8*x + 10

        This should be very fast::

            sage: R.<x> = Integers(next_prime(10^101)*next_prime(10^100))[]             # needs sage.libs.pari
            sage: x / R.base_ring()(2)                                                  # needs sage.libs.pari
            500000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000013365000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000401*x
        """
        return self

    @cached_method
    def multiplicative_group_is_cyclic(self):
        """
        Return ``True`` if the multiplicative group of this field is cyclic.
        This is the case exactly when the order is less than 8, a power
        of an odd prime, or twice a power of an odd prime.

        EXAMPLES::

            sage: R = Integers(7); R
            Ring of integers modulo 7
            sage: R.multiplicative_group_is_cyclic()
            True
            sage: R = Integers(9)
            sage: R.multiplicative_group_is_cyclic()                                    # needs sage.libs.pari
            True
            sage: Integers(8).multiplicative_group_is_cyclic()
            False
            sage: Integers(4).multiplicative_group_is_cyclic()
            True
            sage: Integers(25*3).multiplicative_group_is_cyclic()                       # needs sage.libs.pari
            False

        We test that :issue:`5250` is fixed::

            sage: Integers(162).multiplicative_group_is_cyclic()                        # needs sage.libs.pari
            True
        """
        n = self.order()
        if n < 8:
            return True

        if n % 4 == 0:
            return False  # know n > 7, so n=4 case not a problem
        if n % 4 == 2:
            n = n // 2

        return n.is_prime_power()

    @cached_method
    def multiplicative_generator(self):
        """
        Return a generator for the multiplicative group of this ring,
        assuming the multiplicative group is cyclic.

        Use the unit_gens function to obtain generators even in the
        non-cyclic case.

        EXAMPLES::

            sage: # needs sage.groups sage.libs.pari
            sage: R = Integers(7); R
            Ring of integers modulo 7
            sage: R.multiplicative_generator()
            3
            sage: R = Integers(9)
            sage: R.multiplicative_generator()
            2
            sage: Integers(8).multiplicative_generator()
            Traceback (most recent call last):
            ...
            ValueError: multiplicative group of this ring is not cyclic
            sage: Integers(4).multiplicative_generator()
            3
            sage: Integers(25*3).multiplicative_generator()
            Traceback (most recent call last):
            ...
            ValueError: multiplicative group of this ring is not cyclic
            sage: Integers(25*3).unit_gens()
            (26, 52)
            sage: Integers(162).unit_gens()
            (83,)
        """
        try:
            return self.__mult_gen
        except AttributeError:
            if self.is_field():
                a = self(self.field().multiplicative_generator())
                self.__mult_gen = a
                return a
            if self.multiplicative_group_is_cyclic():
                v = self.unit_gens()
                if len(v) != 1:
                    raise ArithmeticError
                return v[0]

            raise ValueError("multiplicative group of this ring is not cyclic")

    def quadratic_nonresidue(self):
        """
        Return a quadratic non-residue in ``self``.

        EXAMPLES::

            sage: R = Integers(17)
            sage: R.quadratic_nonresidue()                                              # needs sage.libs.pari
            3
            sage: R(3).is_square()
            False
        """
        try:
            return self._nonresidue
        except AttributeError:
            for a in self:
                if not a.is_square():
                    self._nonresidue = a
                    return a

    def square_roots_of_one(self):
        """
        Return all square roots of 1 in self, i.e., all solutions to
        `x^2 - 1 = 0`.

        OUTPUT: the square roots of 1 in ``self`` as a tuple

        EXAMPLES::

            sage: R = Integers(2^10)
            sage: [x for x in R if x^2 == 1]
            [1, 511, 513, 1023]
            sage: R.square_roots_of_one()
            (1, 511, 513, 1023)

        ::

            sage: # needs sage.libs.pari
            sage: v = Integers(9*5).square_roots_of_one(); v
            (1, 19, 26, 44)
            sage: [x^2 for x in v]
            [1, 1, 1, 1]
            sage: v = Integers(9*5*8).square_roots_of_one(); v
            (1, 19, 71, 89, 91, 109, 161, 179, 181, 199, 251, 269, 271, 289, 341, 359)
            sage: [x^2 for x in v]
            [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1]
        """
        try:
            return self.__square_roots_of_one
        except AttributeError:
            pass
        n = self.__order
        if n.is_prime_power():
            if n % 2 == 0:
                # power of 2
                if n == 2:
                    v = [self(1)]
                elif n == 4:
                    v = [self(1), self(3)]
                else:  # n >= 8
                    half_ord = n // 2
                    v = [self(1), self(-1),
                         self(half_ord - 1), self(half_ord + 1)]
            else:
                v = [self(1), self(-1)]
        else:
            # Reduce to the prime power case.
            F = self.factored_order()
            vmod = []
            moduli = []
            for p, e in F:
                k = p**e
                R = IntegerModRing(p**e)
                w = [self(x) for x in R.square_roots_of_one()]
                vmod.append(w)
                moduli.append(k)
            # Now combine in all possible ways using the CRT
            basis = CRT_basis(moduli)
            from sage.misc.mrange import cartesian_product_iterator
            v = []
            for x in cartesian_product_iterator(vmod):
                # x is a specific choice of roots modulo each prime power divisor
                a = sum([basis[i] * x[i] for i in range(len(x))])
                v.append(a)

        v.sort()
        v = tuple(v)
        self.__square_roots_of_one = v
        return v

    @cached_method
    def factored_order(self):
        """
        EXAMPLES::

            sage: R = IntegerModRing(18)
            sage: FF = IntegerModRing(17)
            sage: R.factored_order()
            2 * 3^2
            sage: FF.factored_order()
            17
        """
        return factor(self.__order, int_=(self.__order < 2**31))

    def factored_unit_order(self):
        r"""
        Return a list of :class:`Factorization` objects, each the factorization
        of the order of the units in a `\ZZ / p^n \ZZ` component of this group
        (using the Chinese Remainder Theorem).

        EXAMPLES::

            sage: R = Integers(8*9*25*17*29)
            sage: R.factored_unit_order()
            [2^2, 2 * 3, 2^2 * 5, 2^4, 2^2 * 7]
        """
        ans = []
        from sage.structure.factorization import Factorization
        for p, e in self.factored_order():
            ans.append(Factorization([(p, e - 1)]) *
                       factor(p - 1, int_=(self.__order < 2**31)))
        return ans

    def characteristic(self):
        """
        EXAMPLES::

            sage: R = IntegerModRing(18)
            sage: FF = IntegerModRing(17)
            sage: FF.characteristic()
            17
            sage: R.characteristic()
            18
        """
        return self.__order

    def _repr_(self):
        """
        String representation.

        EXAMPLES::

            sage: Zmod(87)
            Ring of integers modulo 87
        """
        return "Ring of integers modulo {}".format(self.__order)

    def _latex_(self):
        r"""
        Latex representation.

        EXAMPLES::

            sage: latex(Zmod(87))
            \ZZ/87\ZZ
        """
        return "\\ZZ/{}\\ZZ".format(self.__order)

    def modulus(self):
        r"""
        Return the polynomial `x - 1` over this ring.

        .. NOTE::

           This function exists for consistency with the finite-field
           modulus function.

        EXAMPLES::

            sage: R = IntegerModRing(18)
            sage: R.modulus()
            x + 17
            sage: R = IntegerModRing(17)
            sage: R.modulus()
            x + 16
        """
        try:
            return self.__modulus
        except AttributeError:
            x = self['x'].gen()
            self.__modulus = x - 1
            return self.__modulus

    def order(self):
        """
        Return the order of this ring.

        EXAMPLES::

            sage: Zmod(87).order()
            87
        """
        return self.__order

    def cardinality(self):
        """
        Return the cardinality of this ring.

        EXAMPLES::

            sage: Zmod(87).cardinality()
            87
        """
        return self.order()

    def _pari_order(self):
        """
        Return the pari integer representing the order of this ring.

        EXAMPLES::

            sage: Zmod(87)._pari_order()                                                # needs sage.libs.pari
            87
        """
        try:
            return self.__pari_order
        except AttributeError:
            self.__pari_order = pari(self.order())
            return self.__pari_order

    def _element_constructor_(self, x):
        """
        TESTS::

            sage: # needs sage.rings.finite_rings
            sage: K2 = GF(2)
            sage: K3 = GF(3)
            sage: K8 = GF(8, 'a')
            sage: K8(5)  # indirect doctest
            1
            sage: K8('a+1')
            a + 1
            sage: K8(K2(1))
            1

        The following test refers to :issue:`6468`::

            sage: class foo_parent(Parent):
            ....:     pass
            sage: class foo(RingElement):
            ....:     def lift(self):
            ....:         raise PariError
            sage: P = foo_parent()
            sage: F = foo(P)
            sage: GF(2)(F)
            Traceback (most recent call last):
            ...
            TypeError: error coercing to finite field

        The following test refers to :issue:`8970`::

            sage: R = Zmod(13); a = R(2)
            sage: a == R(gap(a))                                                        # needs sage.libs.gap
            True

        libgap interface (:issue:`23714`)::

            sage: a = libgap.eval("Z(13)^2")                                            # needs sage.libs.gap
            sage: a.sage()                                                              # needs sage.libs.gap
            4
            sage: libgap(a.sage()) == a                                                 # needs sage.libs.gap
            True

        better syntax for libgap interface::

            sage: a = libgap.Z(13)^2                                                    # needs sage.libs.gap
            sage: libgap(a.sage()) == a                                                 # needs sage.libs.gap
            True
        """
        try:
            return integer_mod.IntegerMod(self, x)
        except (NotImplementedError, PariError):
            raise TypeError("error coercing to finite field")
        except TypeError:
            if isinstance(x, GapElement):
                from sage.libs.gap.libgap import libgap
                return libgap(x).sage()
            raise  # Continue up with the original TypeError

    def __iter__(self):
        """
        EXAMPLES::

            sage: R = IntegerModRing(3)
            sage: for i in R:
            ....:     print(i)
            0
            1
            2
            sage: L = [i for i in R]
            sage: L[0].parent()
            Ring of integers modulo 3
        """
        i = 0
        order = int(self.__order)
        while i < order:
            yield self(i)
            i = i + 1

    def _coerce_map_from_(self, S):
        r"""
        EXAMPLES::

            sage: R = Integers(15)
            sage: f = R.coerce_map_from(Integers(450)); f  # indirect doctest
            Natural morphism:
              From: Ring of integers modulo 450
              To:   Ring of integers modulo 15
            sage: f(-1)
            14
            sage: f = R.coerce_map_from(int); f
            Native morphism:
              From: Set of Python objects of class 'int'
              To:   Ring of integers modulo 15
            sage: f(-1r)
            14
            sage: f = R.coerce_map_from(ZZ); f
            Natural morphism:
              From: Integer Ring
              To:   Ring of integers modulo 15
            sage: f(-1)
            14
            sage: f = R.coerce_map_from(Integers(10)); print(f)
            None
            sage: f = R.coerce_map_from(QQ); print(f)
            None

            sage: R = IntegerModRing(17)
            sage: a = R(3)
            sage: b = R.coerce(3)
            sage: b
            3
            sage: a==b
            True

        This is allowed::

            sage: R(2/3)
            12

        But this is not, since there is no (canonical or not!) ring
        homomorphism from `\QQ` to `\GF{17}`.

        ::

            sage: R.coerce(2/3)
            Traceback (most recent call last):
            ...
            TypeError: no canonical coercion from Rational Field to Ring of integers modulo 17

        We do not allow the coercion ``GF(p) -> Z/pZ``, because in case of a
        canonical isomorphism, there is a coercion map in only one
        direction, i.e., to the object in the smaller category.
        """
        if S is int:
            return integer_mod.Int_to_IntegerMod(self)
        elif S is integer_ring.ZZ:
            return integer_mod.Integer_to_IntegerMod(self)
        elif isinstance(S, IntegerModRing_generic):
            if isinstance(S, Field):
                return None
            try:
                return integer_mod.IntegerMod_to_IntegerMod(S, self)
            except TypeError:
                pass
        to_ZZ = integer_ring.ZZ._internal_coerce_map_from(S)
        if to_ZZ is not None:
            return integer_mod.Integer_to_IntegerMod(self) * to_ZZ

    def _convert_map_from_(self, other):
        """
        Conversion from `p`-adic fields.

        EXAMPLES::

            sage: Zmod(81).convert_map_from(Qp(3))                                      # needs sage.rings.padics
            Reduction morphism:
              From: 3-adic Field with capped relative precision 20
              To:   Ring of integers modulo 81
        """
        from sage.rings.padics.padic_generic import pAdicGeneric, ResidueReductionMap
        if isinstance(other, pAdicGeneric) and other.degree() == 1:
            p = other.prime()
            N = self.cardinality()
            n = N.exact_log(p)
            if p**n == N:
                return ResidueReductionMap._create_(other, self)

    def __richcmp__(self, other, op):
        """
        EXAMPLES::

            sage: Z11 = IntegerModRing(11); Z11
            Ring of integers modulo 11
            sage: Z12 = IntegerModRing(12); Z12
            Ring of integers modulo 12
            sage: Z13 = IntegerModRing(13); Z13
            Ring of integers modulo 13
            sage: Z11 == Z11, Z11 == Z12, Z11 == Z13
            (True, False, False)
            sage: F = GF(11); F
            Finite Field of size 11
            sage: Z11 == F
            False

        In :issue:`15229`, the following was implemented::

            sage: R1 = IntegerModRing(5)
            sage: R2 = IntegerModRing(5, is_field=True)
            sage: R1 is R2    # used to return False
            True
            sage: R2 == GF(5)
            False
        """
        # We want that GF(p) and IntegerModRing(p) evaluate unequal.
        # However, we cannot just compare the types, since the
        # choice of a different category also changes the type.
        # But if we go to the base class, we avoid the influence
        # of the category.
        try:
            c = bool(other.__class__.__base__ != self.__class__.__base__)
        except AttributeError:  # __base__ does not always exists
            c = bool(type(other) is not type(self))
        if c:
            return NotImplemented
        return richcmp(self.__order, other.__order, op)

    def unit_gens(self, **kwds):
        r"""
        Return generators for the unit group `(\ZZ/N\ZZ)^*`.

        We compute the list of generators using a deterministic algorithm, so
        the generators list will always be the same. For each odd prime divisor
        of `N` there will be exactly one corresponding generator; if `N` is
        even there will be 0, 1 or 2 generators according to whether 2 divides
        `N` to order 1, 2 or `\geq 3`.

        OUTPUT: a tuple containing the units of ``self``

        EXAMPLES::

            sage: R = IntegerModRing(18)
            sage: R.unit_gens()                                                         # needs sage.groups
            (11,)
            sage: R = IntegerModRing(17)
            sage: R.unit_gens()                                                         # needs sage.groups
            (3,)
            sage: IntegerModRing(next_prime(10^30)).unit_gens()                         # needs sage.groups
            (5,)

        The choice of generators is affected by the optional keyword
        ``algorithm``; this can be ``'sage'`` (default) or ``'pari'``.
        See :meth:`unit_group` for details. ::

            sage: A = Zmod(55)
            sage: A.unit_gens(algorithm='sage')                                         # needs sage.groups
            (12, 46)
            sage: A.unit_gens(algorithm='pari')                                         # needs sage.groups sage.libs.pari
            (2, 21)

        TESTS::

            sage: IntegerModRing(2).unit_gens()                                         # needs sage.groups
            ()
            sage: IntegerModRing(4).unit_gens()                                         # needs sage.groups
            (3,)
            sage: IntegerModRing(8).unit_gens()                                         # needs sage.groups
            (7, 5)
        """
        return self.unit_group(**kwds).gens_values()

    def unit_group_exponent(self):
        """
        EXAMPLES::

            sage: R = IntegerModRing(17)
            sage: R.unit_group_exponent()                                               # needs sage.groups
            16
            sage: R = IntegerModRing(18)
            sage: R.unit_group_exponent()                                               # needs sage.groups
            6
        """
        return self.unit_group().exponent()

    def unit_group_order(self):
        """
        Return the order of the unit group of this residue class ring.

        EXAMPLES::

            sage: R = Integers(500)
            sage: R.unit_group_order()                                                  # needs sage.groups
            200
        """
        return self.unit_group().order()

    @cached_method
    def unit_group(self, algorithm='sage'):
        r"""
        Return the unit group of ``self``.

        INPUT:

        - ``self`` -- the ring `\ZZ/n\ZZ` for a positive integer `n`

        - ``algorithm`` -- either ``'sage'`` (default) or ``'pari'``

        OUTPUT:

        The unit group of ``self``.  This is a finite Abelian group
        equipped with a distinguished set of generators, which is
        computed using a deterministic algorithm depending on the
        ``algorithm`` parameter.

        - If ``algorithm == 'sage'``, the generators correspond to the
          prime factors `p \mid n` (one generator for each odd `p`;
          the number of generators for `p = 2` is 0, 1 or 2 depending
          on the order to which 2 divides `n`).

        - If ``algorithm == 'pari'``, the generators are chosen such
          that their orders form a decreasing sequence with respect to
          divisibility.

        EXAMPLES:

        The output of the algorithms ``'sage'`` and ``'pari'`` can
        differ in various ways.  In the following example, the same
        cyclic factors are computed, but in a different order::

            sage: # needs sage.groups
            sage: A = Zmod(15)
            sage: G = A.unit_group(); G
            Multiplicative Abelian group isomorphic to C2 x C4
            sage: G.gens_values()
            (11, 7)
            sage: H = A.unit_group(algorithm='pari'); H                                 # needs sage.libs.pari
            Multiplicative Abelian group isomorphic to C4 x C2
            sage: H.gens_values()                                                       # needs sage.libs.pari
            (7, 11)

        Here are two examples where the cyclic factors are isomorphic,
        but are ordered differently and have different generators::

            sage: # needs sage.groups
            sage: A = Zmod(40)
            sage: G = A.unit_group(); G
            Multiplicative Abelian group isomorphic to C2 x C2 x C4
            sage: G.gens_values()
            (31, 21, 17)
            sage: H = A.unit_group(algorithm='pari'); H                                 # needs sage.libs.pari
            Multiplicative Abelian group isomorphic to C4 x C2 x C2
            sage: H.gens_values()                                                       # needs sage.libs.pari
            (17, 31, 21)

            sage: # needs sage.groups
            sage: A = Zmod(192)
            sage: G = A.unit_group(); G
            Multiplicative Abelian group isomorphic to C2 x C16 x C2
            sage: G.gens_values()
            (127, 133, 65)
            sage: H = A.unit_group(algorithm='pari'); H                                 # needs sage.libs.pari
            Multiplicative Abelian group isomorphic to C16 x C2 x C2
            sage: H.gens_values()                                                       # needs sage.libs.pari
            (133, 127, 65)

        In the following examples, the cyclic factors are not even
        isomorphic::

            sage: A = Zmod(319)
            sage: A.unit_group()                                                        # needs sage.groups
            Multiplicative Abelian group isomorphic to C10 x C28
            sage: A.unit_group(algorithm='pari')                                        # needs sage.groups sage.libs.pari
            Multiplicative Abelian group isomorphic to C140 x C2

            sage: A = Zmod(30.factorial())
            sage: A.unit_group()                                                        # needs sage.groups
            Multiplicative Abelian group isomorphic to
             C2 x C16777216 x C3188646 x C62500 x C2058 x C110 x C156 x C16 x C18 x C22 x C28
            sage: A.unit_group(algorithm='pari')                                        # needs sage.groups sage.libs.pari
            Multiplicative Abelian group isomorphic to
             C20499647385305088000000 x C55440 x C12 x C12 x C4 x C2 x C2 x C2 x C2 x C2 x C2

        TESTS:

        We test the cases where the unit group is trivial::

            sage: # needs sage.groups
            sage: A = Zmod(1)
            sage: A.unit_group()
            Trivial Abelian group
            sage: A.unit_group(algorithm='pari')                                        # needs sage.libs.pari
            Trivial Abelian group
            sage: A = Zmod(2)
            sage: A.unit_group()
            Trivial Abelian group
            sage: A.unit_group(algorithm='pari')                                        # needs sage.libs.pari
            Trivial Abelian group

            sage: Zmod(3).unit_group(algorithm='bogus')                                 # needs sage.groups
            Traceback (most recent call last):
            ...
            ValueError: unknown algorithm 'bogus' for computing the unit group
        """
        from sage.groups.abelian_gps.values import AbelianGroupWithValues
        if algorithm == 'sage':
            n = self.order()
            gens = []
            orders = []
            for p, r in self.factored_order():
                m = n // (p**r)
                for g, o in _unit_gens_primepowercase(p, r):
                    x = g.crt(integer_mod.Mod(1, m))
                    gens.append(x)
                    orders.append(o)
        elif algorithm == 'pari':
            _, orders, gens = self.order().__pari__().znstar()
            gens = [self(g) for g in gens]
            orders = [integer.Integer(o) for o in orders]
        else:
            raise ValueError('unknown algorithm %r for computing the unit group' % algorithm)
        return AbelianGroupWithValues(gens, orders, values_group=self)

    def random_element(self, bound=None):
        """
        Return a random element of this ring.

        INPUT:

        - ``bound`` -- positive integer or ``None`` (the default); if given,
          return  the coercion of an integer in the interval
          ``[-bound, bound]`` into this ring

        EXAMPLES::

            sage: R = IntegerModRing(18)
            sage: R.random_element().parent() is R
            True
            sage: found = [False]*18
            sage: while not all(found):
            ....:     found[R.random_element()] = True

        We test ``bound``-option::

            sage: R.random_element(2) in [R(16), R(17), R(0), R(1), R(2)]
            True
        """
        if bound is not None:
            return CommutativeRing.random_element(self, bound)
        a = random.randint(0, self.order() - 1)
        return self(a)

    @staticmethod
    def _lift_residue_field_root(p, e, f, fprime, root):
        """Lifts a root of f

        INPUT:

        - ``p`` -- integer, a prime number
        - ``e`` -- positive integer
        - ``f`` -- polynomial with coefficients in ``IntegerModRing(p**e)``
        - ``fprime`` -- derivative of ``f``
        - ``root`` -- Element of ``IntegerModRing(p)`` with ``f(root) = 0``

        OUTPUT: iterable of roots of ``f`` modulo ``p**e``.  Each root is an
        ``IntegerModRing(p**e)`` element.

        TESTS::

            sage: R = Zmod(2)
            sage: S.<x> = R[]
            sage: R._lift_residue_field_root(2, 1, S.zero(), S.zero(), R(0))
            (0,)

        Lifting roots of the zero polynomial::

            sage: R = Zmod(41)
            sage: S.<x> = R[]
            sage: R._lift_residue_field_root(41, 1, S.zero(), S.zero(), R(12))
            (12,)
            sage: R = Zmod(5**2)
            sage: S.<x> = R[]
            sage: R._lift_residue_field_root(5, 2, S.zero(), S.zero(), R(2))
            [2, 7, 12, 17, 22]
            sage: R = Zmod(2**3)
            sage: S.<x> = R[]
            sage: R._lift_residue_field_root(2, 3, S.zero(), S.zero(), R(1))
            [1, 5, 3, 7]

        Trivial case where ``e == 1``::

            sage: R = Zmod(41)
            sage: S.<x> = R[]
            sage: f = x^2 - 2
            sage: R._lift_residue_field_root(41, 1, f, f.derivative(), R(17))
            (17,)

            sage: R = Zmod(43)
            sage: S.<x> = R[]
            sage: f = x^43 - 3
            sage: R._lift_residue_field_root(43, 1, f, f.derivative(), R(3))
            (3,)

        Non-singular cases with one step of lifting::

            sage: R = Zmod(2**2)
            sage: S.<x> = R[]
            sage: f = x - 1
            sage: R._lift_residue_field_root(2, 2, f, f.derivative(), R(1))
            (1,)
            sage: f = x - 3
            sage: R._lift_residue_field_root(2, 2, f, f.derivative(), R(1))
            (3,)

            sage: # needs sage.libs.pari
            sage: R = Zmod(4001**2)
            sage: S.<x> = R[]
            sage: f = x^3 - 2
            sage: R._lift_residue_field_root(4001, 2, f, f.derivative(), R(3981))
            (5309307,)
            sage: f = x^3 - 3
            sage: R._lift_residue_field_root(4001, 2, f, f.derivative(), R(1091))
            (11035849,)

        Non-singular cases with multiple steps of lifting::

            sage: R = Zmod(2**10)
            sage: S.<x> = R[]
            sage: f = x + 1
            sage: R._lift_residue_field_root(2, 10, f, f.derivative(), Zmod(2)(1))
            (1023,)

            sage: R = Zmod(2**16)
            sage: S.<x> = R[]
            sage: f = x + 1
            sage: R._lift_residue_field_root(2, 16, f, f.derivative(), Zmod(2)(1))
            (65535,)

            sage: # needs sage.libs.pari
            sage: R = Zmod(7**4)
            sage: S.<x> = R[]
            sage: f = x^4 - 2
            sage: R._lift_residue_field_root(7, 4, f, f.derivative(), Zmod(7)(2))
            (121,)

        Singular cases::

            sage: R = Zmod(2**3)
            sage: S.<x> = R[]
            sage: f = x^2 - 1
            sage: R._lift_residue_field_root(2, 3, f, f.derivative(), Zmod(2)(1))
            [1, 5, 3, 7]
            sage: f = 2*x
            sage: R._lift_residue_field_root(2, 3, f, f.derivative(), Zmod(2)(0))
            [0, 4]

            sage: R = Zmod(11**2)
            sage: S.<x> = R[]
            sage: f = x^2 + 13*x + 1
            sage: R._lift_residue_field_root(11, 2, f, f.derivative(), Zmod(11)(10))
            []

            sage: # needs sage.libs.pari
            sage: R = Zmod(11**3)
            sage: S.<x> = R[]
            sage: f = x^2 + 123*x + 1
            sage: R._lift_residue_field_root(11, 3, f, f.derivative(), Zmod(11)(10))
            [10, 131, 252, 373, 494, 615, 736, 857, 978, 1099, 1220, 109, 230, 351, 472, 593, 714, 835, 956, 1077, 1198, 1319]
        """
        if e == 1:
            # Nothing to do
            return (root,)
        deriv = fprime(root)
        if deriv:
            # Unique lift, use Newton iteration
            prec = 1
            while True:
                prec = min(2*prec, e)
                Zp_prec = Zmod(p**prec)
                root = Zp_prec(root.lift())
                deriv = fprime(root)
                step = f(root) / deriv
                root -= step
                if prec >= e:
                    return (root,)
        else:
            # Non-unique lift, go one power at a time
            prec = 1
            new_power = 1
            new_mod = p
            current_roots = (root,)
            for _ in range(e - 1):
                prec += 1
                new_power = new_mod
                new_mod *= p
                new_roots = []
                Zp_prec = Zmod(new_mod)
                for rt in current_roots:
                    rt = Zp_prec(rt.lift())
                    if f(rt):
                        continue
                    new_roots.append(rt)
                    for _ in range(p - 1):
                        rt += new_power
                        new_roots.append(rt)
                current_roots = new_roots

            return current_roots

    def _roots_univariate_polynomial(self, f, ring=None, multiplicities=True, algorithm=None):
        r"""
        Return the roots of ``f`` in the ring ``ring``.

        INPUT:

        - ``f`` - a polynomial defined over this ring

        - ``ring`` - the ring to find roots in.  Control flow elsewhere
          ensures that the only cases we need to handle are ``self`` and
          ``None``.  Otherwise we raise ``NotImplementedError``.

        - ``multiplicities`` - bool (default: ``True``). If ``True``, return
          list of pairs `(r, n)`, where `r` is a root and `n` is its
          multiplicity. If ``False``, just return the unique roots, with no
          information about multiplicities. Multiplicities are only defined
          over fields, and this method raises ``NotImplementedError`` if this
          is ``True`` but the ring is not a field.

        - ``algorithm`` - ignored

        ALGORITHM:

        The algorithm is adapted from [Gou2020]_, section 4.5, and [Coh1993]_,
        section 3.5.3.  It is a combination of the Chinese Remainder Theorem
        and Hensel's lemma.  As a base case, if `N` is prime, then we find
        roots by factoring `f`.  If `N` is a prime power `p^e`, then we find
        roots modulo `p` and lift them.  Finally, for general `N`, we first
        factor the modulus `N` into prime powers, list all roots modulo those
        prime powers, and combine the roots using the Chinese Remainder
        Theorem.

        Suppose that we are trying to find roots modulo `p^e` and that `r` is
        a root of `f(x)` modulo `p`.  The easy case is when `f'(r) \not\equiv
        0 \pmod{p}`, for then Hensel's lemma implies that there is a unique
        `r_e \in \Zmod{p^e}` with `r_e \equiv r \pmod{p}`.
        Moreover, this `r_e` can be found by applying Newton's method for
        numerically approximating roots.  Each iteration of Newton's method
        doubles the precision to which the root is known.

        But if `f'(r) \equiv 0 \pmod{p}`, then this is no longer true.  In
        fact, in this case roots modulo `p^e` are not the same as `p`-adic
        roots, and finding all the latter does not guarantee that we have
        found all the former.  For example, if `f(x) = 2x` and `p = 2`, then
        there is only one `p`-adic root, namely zero.  But the solutions of
        `2x \equiv 0 \pmod{2^k}` are `0` and `2^{k-1}`; the former lifts to
        two roots modulo `2^{k+1}`, namely `0` and `2^k`, while the latter
        does not lift at all.  We handle this case by lifting one power at a
        time.  While we can no longer use Newton's method to solve for a lift,
        the Taylor series it is based on still yields constraints on the roots
        modulo `p^{k+1}`:  If `r_k` is a root of `f` modulo `p^k`, then either
        every lift of `r_k` to `\Zmod{p^{k + 1}}` is a root of `f`
        modulo `p^{k+1}` or none of them are.  Consequently we may find roots
        modulo `p^e` by lifting one power at a time.

        When `f'(r) \equiv 0 \pmod{p}`, an alternative approach is to change
        variables, factor out the root, and then factor out powers of `p`.
        This has the advantage that it will eventually reach a situation where
        the lift converges quadratically, but it is not presently implemented.
        A different form of Hensel's lemma applies once we are close enough to
        a `p`-adic root (see [Gou2020]_, problem 120), but it seems delicate
        to use it directly to find all roots modulo `p^e` (consider our
        earlier example of `f(x) = 2x`), so we do not presently attempt to
        apply Hensel's lemma in this way.

        EXAMPLES::

            sage: # needs sage.libs.pari
            sage: R.<x> = Zmod(41)[]
            sage: (x^3 + x).roots()
            [(0, 1), (32, 1), (9, 1)]
            sage: (x^3 + x).roots(multiplicities=False)
            [0, 32, 9]
            sage: (x^6 + x^5 + 9*x^4 + 20*x^3 + 3*x^2 + 18*x + 7).roots()
            [(19, 1), (20, 2), (21, 3)]
            sage: (x^6 + x^5 + 9*x^4 + 20*x^3 + 3*x^2 + 18*x + 7).roots(multiplicities=False)
            [19, 20, 21]

        We can find roots without multiplicities over a ring whose modulus is
        a prime power, even a big power:

            sage: # needs sage.libs.pari
            sage: R.<x> = Zmod(7^3)[]
            sage: (x^2 + x + 1).roots(multiplicities=False)
            [18, 324]
            sage: R.<x> = Zmod(2^50)[]
            sage: (x + 1).roots(multiplicities=False)
            [1125899906842623]

        We can also find roots without multiplicities over a ring whose modulus
        is a product of primes or prime powers:

            sage: R.<x> = Zmod(60)[]
            sage: (x^2 - 1).roots(multiplicities=False)                                 # needs sage.libs.pari
            [29, 41, 49, 1, 59, 11, 19, 31]

        We may also ask for roots modulo a quotient of the ring over which the
        polynomial is defined:

            sage: R.<x> = Zmod(120)[]
            sage: (x^2 - 1).roots(multiplicities=False)                                 # needs sage.libs.pari
            [89, 41, 49, 1, 29, 101, 109, 61, 59, 11, 19, 91, 119, 71, 79, 31]
            sage: (x^2 - 1).roots(Zmod(60), multiplicities=False)                       # needs sage.libs.pari
            [29, 41, 49, 1, 59, 11, 19, 31]

        TESTS::

            sage: R.<x> = Zmod(2)[]
            sage: x.roots()                                                             # needs sage.libs.pari
            [(0, 1)]

        Test polynomials with content:

            sage: R.<x> = Zmod(4)[]
            sage: (2*x).roots(multiplicities=False)                                     # needs sage.libs.pari
            [0, 2]

            sage: R.<x> = Zmod(6)[]
            sage: (3*x).roots(multiplicities=False)                                     # needs sage.libs.pari
            [0, 4, 2]

        Test polynomial with many roots:

            sage: R.<x> = Zmod(6)[]
            sage: f = x * (x - 1) * (x - 2) * (x - 3) * (x - 4) * (x - 5)
            sage: len(f.roots(multiplicities=False))                                    # needs sage.libs.pari
            6

        Test finding roots over large prime powers:

            sage: R.<x> = Zmod(2**16)[]
            sage: (x^3 + 5).roots(multiplicities=False)                                 # needs sage.libs.pari
            [45475]
            sage: (x^2 + 46*x + 1).roots(multiplicities=False)                          # needs sage.libs.pari
            [421, 33189, 16805, 49573, 8613, 41381, 24997, 57765, 7725, 40493, 24109, 56877, 15917, 48685, 32301, 65069]

            sage: # needs sage.libs.pari
            sage: R.<x> = Zmod(3**16)[]
            sage: (x^2 + 2).roots(multiplicities=False)
            [24620738, 18425983]
            sage: (x^2 + 11*x + 1).roots(multiplicities=False)
            [633836, 14982743, 29331650, 13715060, 28063967, 42412874]
            sage: (x^3 + 8).roots(multiplicities=False)
            [14348905, 28697812, 43046719]

        Test some larger primes:

            sage: # needs sage.libs.pari
            sage: R.<x> = Zmod(41**4)[]
            sage: (x^2 + 2).roots(multiplicities=False)
            [2208905, 616856]
            sage: R.<x> = Zmod(43**4)[]
            sage: (x^2 + 3).roots(multiplicities=False)
            [3269879, 148922]

        We can't find roots with multiplicities in non-fields:

            sage: # needs sage.libs.pari
            sage: R.<x> = Zmod(6)[]
            sage: (x + 1).roots()
            Traceback (most recent call last):
            ...
            NotImplementedError: root finding with multiplicities for this polynomial not implemented (try the multiplicities=False option)
            sage: R.<x> = Zmod(8)[]
            sage: (x + 1).roots()
            Traceback (most recent call last):
            ...
            NotImplementedError: root finding with multiplicities for this polynomial not implemented (try the multiplicities=False option)
            sage: R.<x> = Zmod(12)[]
            sage: (x + 1).roots()
            Traceback (most recent call last):
            ...
            NotImplementedError: root finding with multiplicities for this polynomial not implemented (try the multiplicities=False option)

        The zero polynomial has every residue class as a root, but we don't
        support multiplicities even over fields (they would all be infinite).

            sage: R.<x> = Zmod(6)[]
            sage: R.zero().roots()
            Traceback (most recent call last):
            ...
            NotImplementedError: root finding with multiplicities for this polynomial not implemented (try the multiplicities=False option)
            sage: R.zero().roots(multiplicities=False)
            [0, 1, 2, 3, 4, 5]

            sage: R.<x> = Zmod(7)[]
            sage: R.zero().roots()
            Traceback (most recent call last):
            ...
            NotImplementedError: root finding with multiplicities for this polynomial not implemented (try the multiplicities=False option)
            sage: R.zero().roots(multiplicities=False)
            [0, 1, 2, 3, 4, 5, 6]

            sage: R.<x> = Zmod(8)[]
            sage: R.zero().roots()
            Traceback (most recent call last):
            ...
            NotImplementedError: root finding with multiplicities for this polynomial not implemented (try the multiplicities=False option)
            sage: R.zero().roots(multiplicities=False)
            [0, 1, 2, 3, 4, 5, 6, 7]

        This method doesn't support root-finding over rings that aren't Z/nZ:

            sage: R.<x> = Zmod(120)[]
            sage: f = x^2 - 1
            sage: f.base_ring()._roots_univariate_polynomial(f, ring=RR, multiplicities=False)
            Traceback (most recent call last):
            ...
            NotImplementedError

        Sage allows us to coerce polynomials from one modulus to another,
        and that makes the following defined:

            sage: # needs sage.libs.pari
            sage: R.<x> = Zmod(100)[]
            sage: (x^2 - 1).roots(Zmod(99), multiplicities=False) == (x^2 - 1).change_ring(Zmod(99)).roots(multiplicities=False)
            True
        """

        # This function only supports roots in an IntegerModRing
        if ring is not self and ring is not None:
            raise NotImplementedError

        deg = f.degree()

        if multiplicities:
            if deg < 0 or not self.is_field():
                raise NotImplementedError(
                    "root finding with multiplicities for this polynomial not"
                    " implemented (try the multiplicities=False option)"
                )
            # Roots of non-zero polynomial over finite fields by factorization
            return f._roots_from_factorization(f.factor(), multiplicities)

        # Zero polynomial is a base case
        if deg < 0:
            # All residue classes are roots of the zero polynomial
            return [*map(self, range(self.cardinality()))]

        # Finite fields are a base case
        if self.is_field():
            return f._roots_from_factorization(f.factor(), False)

        # Otherwise, find roots modulo each prime power
        fac = self.factored_order()
        prime_power_roots = []
        for p, e in fac:
            Zpe = Zmod(p**e)
            fpe = f.change_ring(Zpe)
            fpe_prime = fpe.derivative()
            fp = fpe.change_ring(Zmod(p))

            mod_p_roots = fp.roots(multiplicities=False)

            this_prime_power = []
            for root in mod_p_roots:
                this_prime_power.extend(
                    self._lift_residue_field_root(p, e, fpe, fpe_prime, root)
                )
            prime_power_roots.append(this_prime_power)

        # Combine using Chinese Remainder Theorem
        ppwr_basis = CRT_basis([p**e for p, e in fac])
        result = []
        for res in cartesian_product_iterator(prime_power_roots):
            root = self.zero()
            for c, x in zip(ppwr_basis, res):
                root += c*x.lift()
            result.append(root)
        return result

    #######################################################
    # Suppose for interfaces
    #######################################################
    def _gap_init_(self):
        """
        EXAMPLES::

            sage: R = Integers(12345678900)
            sage: R
            Ring of integers modulo 12345678900
            sage: gap(R)  # indirect doctest                                            # needs sage.libs.gap
            (Integers mod 12345678900)
        """
        return 'ZmodnZ({})'.format(self.order())

    def _magma_init_(self, magma):
        """
        EXAMPLES::

            sage: R = Integers(12345678900)
            sage: R
            Ring of integers modulo 12345678900
            sage: magma(R)  # indirect doctest, optional - magma
            Residue class ring of integers modulo 12345678900
        """
        return 'Integers({})'.format(self.order())

    def degree(self):
        """
        Return 1.

        EXAMPLES::

            sage: R = Integers(12345678900)
            sage: R.degree()
            1
        """
        return integer.Integer(1)


# Register unpickling methods for backward compatibility.

from sage.misc.persist import register_unpickle_override
register_unpickle_override('sage.rings.integer_mod_ring', 'IntegerModRing_generic', IntegerModRing_generic)


def crt(v):
    """
    INPUT:

    - ``v`` -- (list) a lift of elements of ``rings.IntegerMod(n)``, for
      various coprime moduli ``n``

    EXAMPLES::

        sage: from sage.rings.finite_rings.integer_mod_ring import crt
        sage: crt([mod(3, 8), mod(1,19), mod(7, 15)])
        1027
    """
    if len(v) == 0:
        return IntegerModRing(1)(1)
    x = v[0]
    for i in range(1, len(v)):
        x = x.crt(v[i])
    return x



# sage_setup: distribution = sagemath-categories
# sage.doctest: needs sage.rings.finite_rings
"""
Base class for finite field elements

AUTHORS:

- David Roe (2010-01-14): factored out of sage.structure.element
- Sebastian Oehms (2018-07-19): added :meth:`conjugate` (see :issue:`26761`)
"""

# ****************************************************************************
#       Copyright (C) 2010 David Roe <roed@math.harvard.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.structure.element cimport Element
from sage.structure.parent cimport Parent
from sage.rings.integer_ring import ZZ
from sage.rings.integer import Integer


def is_FiniteFieldElement(x):
    """
    Return ``True`` if ``x`` is a finite field element.

    This function is deprecated.

    EXAMPLES::

        sage: from sage.rings.finite_rings.element_base import is_FiniteFieldElement
        sage: is_FiniteFieldElement(1)
        doctest:...: DeprecationWarning: the function is_FiniteFieldElement is deprecated; use isinstance(x, sage.structure.element.FieldElement) and x.parent().is_finite() instead
        See https://github.com/sagemath/sage/issues/32664 for details.
        False
        sage: is_FiniteFieldElement(IntegerRing())
        False
        sage: is_FiniteFieldElement(GF(5)(2))
        True
    """
    from sage.misc.superseded import deprecation
    deprecation(32664, "the function is_FiniteFieldElement is deprecated; use isinstance(x, sage.structure.element.FieldElement) and x.parent().is_finite() instead")

    from sage.rings.finite_rings.finite_field_base import FiniteField
    return isinstance(x, Element) and isinstance(x.parent(), FiniteField)


cdef class FiniteRingElement(CommutativeRingElement):
    def _nth_root_common(self, n, all, algorithm, cunningham):
        """
        This function exists to reduce code duplication between finite field
        `n`-th roots and ``integer_mod`` `n`-th roots. It assumes that ``self``
        is a field element.

        The inputs are described there.

        TESTS::

            sage: a = Zmod(17)(13)
            sage: sorted(a._nth_root_common(4, True, "Johnston", False))
            [3, 5, 12, 14]
            sage: sorted(a._nth_root_common(4, True, "Johnston", cunningham=True))  # optional - cunningham_tables
            [3, 5, 12, 14]

        Test various prime powers::

            sage: p = 5^5*10000000100 + 1
            sage: a = GF(p)(3)**(5^7)
            sage: for e in range(20):
            ....:     r = a._nth_root_common(5^e, False, "Johnston", False)
            ....:     assert r**(5^e) == a

        Test very large modulus (assumed impossible to factor in reasonable time)::

            sage: p = 2^1024 + 643
            sage: a = GF(p, proof=False)(3)**(29*283*3539)
            sage: r = a._nth_root_common(29*283*3539*12345, False, "Johnston", False)
            sage: r**(29*283*3539*12345) == a
            True
        """
        K = self.parent()
        q = K.order()
        gcd = n.gcd(q-1)
        if self.is_one():
            if gcd == 1:
                if all: return [self]
                else: return self
            else:
                nthroot = K.zeta(gcd)
                return [nthroot**a for a in range(gcd)] if all else nthroot
        if gcd == q-1:
            if all: return []
            else: raise ValueError("no nth root")
        gcd, alpha, _ = n.xgcd(q-1)  # gcd = alpha*n + beta*(q-1), so 1/n = alpha/gcd (mod q-1)
        if gcd == 1:
            return [self**alpha] if all else self**alpha

        n = gcd
        q1overn = (q-1)//n
        if self**q1overn != 1:
            if all: return []
            else: raise ValueError("no nth root")
        self = self**alpha
        if cunningham:
            from sage.rings.factorint import factor_cunningham
            F = factor_cunningham(n)
        else:
            F = n.factor()
        from sage.groups.generic import discrete_log
        if algorithm is None or algorithm == 'Johnston':
            # In the style of the Adleman-Manders-Miller algorithm,
            # we will use small order elements instead of a multiplicative
            # generator, which can be expensive to compute.
            for r, v in F:
                # 0 < v <= k
                k, h = (q-1).val_unit(r)
                hinv = (-h).inverse_mod(r**v)
                z = h * hinv
                x = (1 + z) // r**v
                if k == v:
                    self = self**x
                else:
                    # We need an element of order r^k (g^h in Johnston's article)
                    # self^x differs from the actual nth root by an element of
                    # order dividing r^(k-v)
                    gh = K.zeta(r**k)
                    t = discrete_log(self**h, gh**(r**v), r**(k-v), operation='*')
                    self = self**x * gh**(-hinv*t)
            if all:
                nthroot = K.zeta(n)
                L = [self]
                for i in range(1,n):
                    self *= nthroot
                    L.append(self)
                return L
            else:
                return self
        else:
            raise ValueError("unknown algorithm")

    def to_bytes(self, byteorder='big'):
        r"""
        Return an array of bytes representing an integer.

        Internally relies on the python ``int.to_bytes()`` method.
        Length of byte array is determined from the field's order.

        INPUT:

        - ``byteorder`` -- string (default: ``'big'``); determines the byte order of
          ``input_bytes``; can only be ``'big'`` or ``'little'``

        EXAMPLES::

            sage: F = GF(65537)
            sage: a = F(8726)
            sage: a.to_bytes()
            b'\x00"\x16'
            sage: a.to_bytes(byteorder='little')
            b'\x16"\x00'
        """
        length = (self.parent().order().nbits() + 7) // 8
        return int(self).to_bytes(length=length, byteorder=byteorder)

cdef class FinitePolyExtElement(FiniteRingElement):
    """
    Elements represented as polynomials modulo a given ideal.

    TESTS::

        sage: k.<a> = GF(64)
        sage: TestSuite(a).run()
    """
    def _im_gens_(self, codomain, im_gens, base_map=None):
        """
        Used for applying homomorphisms of finite fields.

        EXAMPLES::

            sage: k.<a> = FiniteField(73^2)
            sage: K.<b> = FiniteField(73^4)
            sage: phi = k.hom([ b^(73*73+1) ]) # indirect doctest
            sage: phi(0)
            0
            sage: phi(a)
            7*b^3 + 13*b^2 + 65*b + 71

            sage: phi(a+3)
            7*b^3 + 13*b^2 + 65*b + 1
        """
        ## NOTE: see the note in sage/rings/number_field_element.pyx,
        ## in the comments for _im_gens_ there -- something analogous
        ## applies here.
        f = self.polynomial()
        if base_map is not None:
            Cx = codomain['x']
            f = Cx([base_map(c) for c in f])
        return codomain(f(im_gens[0]))

    def minpoly(self, var='x', algorithm='pari'):
        """
        Return the minimal polynomial of this element
        (over the corresponding prime subfield).

        INPUT:

        - ``var`` -- string (default: ``'x'``)

        - ``algorithm`` -- string (default: ``'pari'``):

          - ``'pari'`` -- use pari's minpoly

          - ``'matrix'`` -- return the minpoly computed from the matrix of
            left multiplication by self

        EXAMPLES::

            sage: from sage.rings.finite_rings.element_base import FinitePolyExtElement
            sage: k.<a> = FiniteField(19^2)
            sage: parent(a)
            Finite Field in a of size 19^2
            sage: b=a**20
            sage: p=FinitePolyExtElement.minpoly(b,"x", algorithm='pari')
            sage: q=FinitePolyExtElement.minpoly(b,"x", algorithm='matrix')
            sage: q == p
            True
            sage: p
            x + 17
        """
        if self.polynomial().degree() == 0:
            from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
            R = PolynomialRing(self.parent().prime_subfield(), var)
            return R.gen() - self.polynomial()[0]

        if algorithm == 'pari':
            from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
            R = PolynomialRing(self.parent().prime_subfield(), var)
            return R(self.__pari__().minpoly('x').lift())
        elif algorithm == 'matrix':
            return self.matrix().minpoly(var)
        else:
            raise ValueError("unknown algorithm '%s'" % algorithm)

    # We have two names for the same method
    # for compatibility with sage.matrix
    def minimal_polynomial(self, var='x'):
        """
        Return the minimal polynomial of this element
        (over the corresponding prime subfield).

        EXAMPLES::

            sage: k.<a> = FiniteField(3^4)
            sage: parent(a)
            Finite Field in a of size 3^4
            sage: b=a**20;p=charpoly(b,"y");p
            y^4 + 2*y^2 + 1
            sage: factor(p)
            (y^2 + 1)^2
            sage: b.minimal_polynomial('y')
            y^2 + 1
        """
        return self.minpoly(var)

    def __getitem__(self, n):
        r"""
        Return the `n`-th coefficient of this finite field element when
        written as a polynomial in the generator.

        EXAMPLES::

            sage: x = polygen(GF(19))
            sage: F.<i> = GF(19^2, modulus=x^2+1)
            sage: a = 5 + 7*i
            sage: a[0]
            5
            sage: a[1]
            7

        ::

            sage: b = F(11)
            sage: b[0]
            11
            sage: b[1]
            0

        TESTS::

            sage: # needs sage.modules
            sage: F,t = GF(random_prime(99)^randrange(2,99), 't').objgen()
            sage: a = F.random_element()
            sage: all(a[i] == a.polynomial()[i] for i in range(F.degree()))
            True
            sage: a == sum(a[i]*t^i for i in range(F.degree()))
            True
        """
        if n < 0 or n >= self.parent().degree():
            raise IndexError("index must lie between 0 and the degree minus 1")
        return self.polynomial()[n]

    def list(self):
        r"""
        Return the list of coefficients (in little-endian) of this
        finite field element when written as a polynomial in the
        generator.

        Equivalent to calling ``list()`` on this element.

        EXAMPLES::

            sage: x = polygen(GF(71))
            sage: F.<u> = GF(71^7, modulus=x^7 + x + 1)
            sage: a = 3 + u + 3*u^2 + 3*u^3 + 7*u^4
            sage: a.list()
            [3, 1, 3, 3, 7, 0, 0]
            sage: a.list() == list(a) == [a[i] for i in range(F.degree())]
            True

        The coefficients returned are those of a fully reduced
        representative of the finite field element::

            sage: b = u^777
            sage: b.list()
            [9, 69, 4, 27, 40, 10, 56]
            sage: (u.polynomial()^777).list()
            [0, 0, 0, 0, ..., 0, 1]

        TESTS::

            sage: # needs sage.modules
            sage: R.<x> = GF(17)[]
            sage: F.<t> = GF(17^60)
            sage: a = F.random_element()
            sage: a == R(a.list())(t)
            True
            sage: list(a) == a.list()
            True
        """
        return self.polynomial().padded_list(self.parent().degree())

    def __iter__(self):
        r"""
        Return an iterator over the coefficients of this finite field
        element, in the same order as :meth:`list`.

        EXAMPLES::

            sage: x = polygen(GF(19))
            sage: F.<i> = GF(19^2, modulus=x^2+1)
            sage: a = 5 + 7*i
            sage: it = iter(a)
            sage: next(it)
            5
            sage: next(it)
            7
            sage: next(it)
            Traceback (most recent call last):
            ...
            StopIteration
            sage: list(a)   # implicit doctest
            [5, 7]
            sage: tuple(a)  # implicit doctest
            (5, 7)
            sage: b = F(11)
            sage: list(b)   # implicit doctest
            [11, 0]
            sage: tuple(b)  # implicit doctest
            (11, 0)
            sage: list(b.polynomial())
            [11]

        TESTS::

            sage: # needs sage.modules
            sage: F = GF(random_prime(333)^randrange(111,999),'t')
            sage: a = F.random_element()
            sage: list(a) == a.list()  # implicit doctest
            True

        ::

            sage: # needs sage.modules
            sage: F.<t> = GF(17^60)
            sage: a = F.random_element()
            sage: a == sum(c*t^i for i,c in enumerate(a))  # implicit doctest
            True

        ::

            sage: # needs sage.modules
            sage: F.<t> = GF((2^127 - 1)^10, 't')
            sage: a = F.random_element()
            sage: a == sum(c*t^i for i,c in enumerate(a))  # implicit doctest
            True
        """
        return iter(self.list())

    def _vector_(self, reverse=False):
        """
        Return a vector matching this element in the vector space attached
        to the parent.  The most significant coefficient is to the right.

        INPUT:

        - ``reverse`` -- reverse the order of the bits
          from little endian to big endian

        EXAMPLES::

            sage: # needs sage.modules
            sage: k.<a> = GF(2^16)
            sage: e = a^2 + 1
            sage: v = vector(e)
            sage: v
            (1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
            sage: k(v)
            a^2 + 1

            sage: # needs sage.modules
            sage: k.<a> = GF(3^16)
            sage: e = 2*a^2 + 1
            sage: v = vector(e)
            sage: v
            (1, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
            sage: k(v)
            2*a^2 + 1

        You can also compute the vector in the other order::

            sage: e._vector_(reverse=True)                                              # needs sage.modules
            (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 1)
        """
        # vector(foo) might pass in ZZ
        if isinstance(reverse, Parent):
            raise TypeError("Base field is fixed to prime subfield.")

        k = self.parent()
        ret = self.polynomial().padded_list(k.degree())

        if reverse:
            ret.reverse()
        return k.vector_space(map=False)(ret)

    def matrix(self, reverse=False):
        r"""
        Return the matrix of left multiplication by the element on
        the power basis `1, x, x^2, \ldots, x^{d-1}` for the field
        extension.

        Thus the \emph{columns} of this matrix give the images
        of each of the `x^i`.

        INPUT:

        - ``reverse`` -- if ``True``, act on vectors in reversed order

        EXAMPLES::

            sage: # needs sage.modules
            sage: k.<a> = GF(2^4)
            sage: b = k.random_element()
            sage: vector(a*b) == a.matrix() * vector(b)
            True
            sage: (a*b)._vector_(reverse=True) == a.matrix(reverse=True) * b._vector_(reverse=True)
            True
        """
        K = self.parent()
        a = K.gen()
        x = K(1)
        d = K.degree()

        columns = []

        for i in range(d):
            columns.append( (self * x)._vector_(reverse=reverse) )
            x *= a

        if reverse:
            columns.reverse()

        from sage.matrix.matrix_space import MatrixSpace
        M = MatrixSpace(K.base_ring(), d)

        return M(columns).transpose()

    def _latex_(self):
        r"""
        Return the latex representation of ``self``, which is just the
        latex representation of the polynomial representation of ``self``.

        EXAMPLES::

            sage: k.<b> = GF(5^2); k
            Finite Field in b of size 5^2
            sage: b._latex_()
            'b'
            sage: (b^2+1)._latex_()
            'b + 4'
        """
        if self.parent().degree()>1:
            return self.polynomial()._latex_()
        else:
            return str(self)

    def __pari__(self, var=None):
        r"""
        Return PARI representation of this finite field element.

        INPUT:

        - ``var`` -- (default: ``None``) optional variable string

        EXAMPLES::

            sage: k.<a> = GF(5^3)
            sage: a.__pari__()
            a
            sage: a.__pari__('b')
            b
            sage: t = 3*a^2 + 2*a + 4
            sage: t_string = t._pari_init_('y')
            sage: t_string
            'Mod(Mod(3, 5)*y^2 + Mod(2, 5)*y + Mod(4, 5), Mod(1, 5)*y^3 + Mod(3, 5)*y + Mod(3, 5))'
            sage: type(t_string)
            <... 'str'>
            sage: t_element = t.__pari__('b')
            sage: t_element
            3*b^2 + 2*b + 4
            sage: type(t_element)
            <class 'cypari2.gen.Gen'>
        """
        if var is None:
            var = self.parent().variable_name()
        ffgen = self._parent.modulus()._pari_with_name(var).ffgen()
        polypari = self.polynomial()._pari_with_name()
        # Add ffgen - ffgen to ensure that we really get an FFELT
        return polypari.subst("x", ffgen) + ffgen - ffgen

    def _pari_init_(self, var=None):
        r"""
        Return a string that defines this element when evaluated in PARI.

        INPUT:

        - ``var`` -- (default: ``None``) a string for a new variable name to use

        EXAMPLES::

            sage: S.<b> = GF(5^2); S
            Finite Field in b of size 5^2
            sage: b._pari_init_()
            'Mod(Mod(1, 5)*b, Mod(1, 5)*b^2 + Mod(4, 5)*b + Mod(2, 5))'
            sage: (2*b+3)._pari_init_()
            'Mod(Mod(2, 5)*b + Mod(3, 5), Mod(1, 5)*b^2 + Mod(4, 5)*b + Mod(2, 5))'

        TESTS:

        The following tests against a bug fixed in :issue:`11530`::

            sage: F.<d> = GF(3^4)
            sage: F.modulus()
            x^4 + 2*x^3 + 2
            sage: d._pari_init_()
            'Mod(Mod(1, 3)*d, Mod(1, 3)*d^4 + Mod(2, 3)*d^3 + Mod(2, 3))'
            sage: (d^2+2*d+1)._pari_init_("p")
            'Mod(Mod(1, 3)*p^2 + Mod(2, 3)*p + Mod(1, 3), Mod(1, 3)*p^4 + Mod(2, 3)*p^3 + Mod(2, 3))'
            sage: d.__pari__()
            d

            sage: K.<M> = GF(2^8)
            sage: K.modulus()
            x^8 + x^4 + x^3 + x^2 + 1
            sage: (M^3+1)._pari_init_()
            'Mod(Mod(1, 2)*M^3 + Mod(1, 2), Mod(1, 2)*M^8 + Mod(1, 2)*M^4 + Mod(1, 2)*M^3 + Mod(1, 2)*M^2 + Mod(1, 2))'
            sage: M._pari_init_(var='foo')
            'Mod(Mod(1, 2)*foo, Mod(1, 2)*foo^8 + Mod(1, 2)*foo^4 + Mod(1, 2)*foo^3 + Mod(1, 2)*foo^2 + Mod(1, 2))'
        """
        if var is None:
            var = self.parent().variable_name()
        g = self.parent().modulus()._pari_with_name(var)
        f = self.polynomial()._pari_with_name(var)
        return 'Mod({0}, {1})'.format(f, g)

    def charpoly(self, var='x', algorithm='pari'):
        """
        Return the characteristic polynomial of ``self`` as a polynomial with given variable.

        INPUT:

        - ``var`` -- string (default: ``'x'``)

        - ``algorithm`` -- string (default: ``'pari'``):

          - ``'pari'`` -- use pari's charpoly

          - ``'matrix'`` -- return the charpoly computed from the matrix of
            left multiplication by ``self``

        The result is not cached.

        EXAMPLES::

            sage: from sage.rings.finite_rings.element_base import FinitePolyExtElement
            sage: k.<a> = FiniteField(19^2)
            sage: parent(a)
            Finite Field in a of size 19^2
            sage: b = a**20
            sage: p = FinitePolyExtElement.charpoly(b, "x", algorithm='pari')
            sage: q = FinitePolyExtElement.charpoly(b, "x", algorithm='matrix')         # needs sage.modules
            sage: q == p                                                                # needs sage.modules
            True
            sage: p
            x^2 + 15*x + 4
            sage: factor(p)
            (x + 17)^2
            sage: b.minpoly('x')
            x + 17
        """
        if algorithm == 'pari':
            from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
            R = PolynomialRing(self.parent().prime_subfield(), var)
            return R(self.__pari__().charpoly('x').lift())
        elif algorithm == 'matrix':
            return self.matrix().charpoly(var)
        else:
            raise ValueError("unknown algorithm '%s'" % algorithm)

    def norm(self):
        """
        Return the norm of ``self`` down to the prime subfield.

        This is the product of the Galois conjugates of ``self``.

        EXAMPLES::

            sage: S.<b> = GF(5^2); S
            Finite Field in b of size 5^2
            sage: b.norm()
            2
            sage: b.charpoly('t')
            t^2 + 4*t + 2

        Next we consider a cubic extension::

            sage: S.<a> = GF(5^3); S
            Finite Field in a of size 5^3
            sage: a.norm()
            2
            sage: a.charpoly('t')
            t^3 + 3*t + 3
            sage: a * a^5 * (a^25)
            2
        """
        f = self.charpoly('x')
        n = f[0]
        if f.degree() % 2:
            return -n
        else:
            return n

    def trace(self):
        """
        Return the trace of this element, which is the sum of the
        Galois conjugates.

        EXAMPLES::

            sage: S.<a> = GF(5^3); S
            Finite Field in a of size 5^3
            sage: a.trace()
            0
            sage: a.charpoly('t')
            t^3 + 3*t + 3
            sage: a + a^5 + a^25
            0
            sage: z = a^2 + a + 1
            sage: z.trace()
            2
            sage: z.charpoly('t')
            t^3 + 3*t^2 + 2*t + 2
            sage: z + z^5 + z^25
            2
        """
        return self.parent().prime_subfield()(self.__pari__().trace().lift())

    def multiplicative_order(self):
        r"""
        Return the multiplicative order of this field element.

        EXAMPLES::

            sage: S.<a> = GF(5^3); S
            Finite Field in a of size 5^3
            sage: a.multiplicative_order()
            124
            sage: (a^8).multiplicative_order()
            31
            sage: S(0).multiplicative_order()
            Traceback (most recent call last):
            ...
            ArithmeticError: Multiplicative order of 0 not defined.
        """
        if self.is_zero():
            raise ArithmeticError("Multiplicative order of 0 not defined.")
        n = self._parent.order() - 1
        F = self._parent.factored_unit_order()[0]
        order = Integer(1)
        for p, e in F:
            # Determine the power of p that divides the order.
            a = self**(n//(p**e))
            while a != 1:
                order = order * p
                a = a**p

        return order

    def additive_order(self):
        """
        Return the additive order of this finite field element.

        EXAMPLES::

            sage: k.<a> = FiniteField(2^12, 'a')
            sage: b = a^3 + a + 1
            sage: b.additive_order()
            2
            sage: k(0).additive_order()
            1
        """
        if self.is_zero():
            return Integer(1)
        return self.parent().characteristic()

    def is_square(self):
        """
        Return ``True`` if and only if this element is a perfect square.

        EXAMPLES::

            sage: k.<a> = FiniteField(9, impl='givaro', modulus='primitive')            # needs sage.libs.linbox
            sage: a.is_square()                                                         # needs sage.libs.linbox
            False
            sage: (a**2).is_square()                                                    # needs sage.libs.linbox
            True
            sage: k.<a> = FiniteField(4, impl='ntl', modulus='primitive')               # needs sage.libs.ntl
            sage: (a**2).is_square()                                                    # needs sage.libs.ntl
            True
            sage: k.<a> = FiniteField(17^5, impl='pari_ffelt', modulus='primitive')     # needs sage.libs.pari
            sage: a.is_square()                                                         # needs sage.libs.pari
            False
            sage: (a**2).is_square()                                                    # needs sage.libs.pari
            True

        ::

            sage: k(0).is_square()                                                      # needs sage.libs.linbox
            True
        """
        K = self.parent()
        if K.characteristic() == 2:
            return True
        n = K.order() - 1
        a = self**(n // 2)
        return a == 1 or a == 0

    def square_root(self, extend=False, all=False):
        """
        The square root function.

        INPUT:

        - ``extend`` -- boolean (default: ``True``); if ``True``, return a
          square root in an extension ring, if necessary. Otherwise, raise a
          :exc:`ValueError` if the root is not in the base ring.

           .. WARNING::

               This option is not implemented!

        - ``all`` -- boolean (default: ``False``); if ``True``, return all
          square roots of ``self``, instead of just one

        .. WARNING::

           The ``'extend'`` option is not implemented (yet).

        EXAMPLES::

            sage: F = FiniteField(7^2, 'a')
            sage: F(2).square_root()
            4
            sage: F(3).square_root()
            2*a + 6
            sage: F(3).square_root()**2
            3
            sage: F(4).square_root()
            2
            sage: K = FiniteField(7^3, 'alpha', impl='pari_ffelt')
            sage: K(3).square_root()
            Traceback (most recent call last):
            ...
            ValueError: must be a perfect square.
        """
        try:
            return self.nth_root(2, extend=extend, all=all)
        except ValueError:
            raise ValueError("must be a perfect square.")

    def sqrt(self, extend=False, all=False):
        """
        See :meth:`square_root`.

        EXAMPLES::

            sage: k.<a> = GF(3^17)
            sage: (a^3 - a - 1).sqrt()
            a^16 + 2*a^15 + a^13 + 2*a^12 + a^10 + 2*a^9 + 2*a^8 + a^7 + a^6 + 2*a^5 + a^4 + 2*a^2 + 2*a + 2
        """
        return self.square_root(extend=extend, all=all)

    def nth_root(self, n, extend=False, all=False, algorithm=None, cunningham=False):
        r"""
        Return an `n`-th root of ``self``.

        INPUT:

        - ``n`` -- integer `\geq 1`

        - ``extend`` -- boolean (default: ``False``); if ``True``, return an
          `n`-th root in an extension ring, if necessary. Otherwise, raise a
          :exc:`ValueError` if the root is not in the base ring.  Warning:
          this option is not implemented!

        - ``all`` -- boolean (default: ``False``); if ``True``, return all `n`-th
          roots of ``self``, instead of just one

        - ``algorithm`` -- string (default: ``None``); ``'Johnston'`` is the
          only currently supported option.  For IntegerMod elements, the problem
          is reduced to the prime modulus case using CRT and `p`-adic logs,
          and then this algorithm used.

        OUTPUT:

        If ``self`` has an `n`-th root, returns one (if ``all`` is ``False``) or a
        list of all of them (if ``all`` is ``True``).
        Otherwise, raises a :exc:`ValueError` (if ``extend`` is ``False``)
        or a :exc:`NotImplementedError` (if ``extend`` is ``True``).

        .. warning::

           The ``extend`` option is not implemented (yet).

        EXAMPLES::

            sage: K = GF(31)
            sage: a = K(22)
            sage: K(22).nth_root(7)
            13
            sage: K(25).nth_root(5)
            5
            sage: K(23).nth_root(3)
            29

            sage: K.<a> = GF(625)
            sage: (3*a^2+a+1).nth_root(13)**13
            3*a^2 + a + 1

            sage: k.<a> = GF(29^2)
            sage: b = a^2 + 5*a + 1
            sage: b.nth_root(11)
            3*a + 20
            sage: b.nth_root(5)
            Traceback (most recent call last):
            ...
            ValueError: no nth root
            sage: b.nth_root(5, all = True)
            []
            sage: b.nth_root(3, all = True)
            [14*a + 18, 10*a + 13, 5*a + 27]

            sage: k.<a> = GF(29^5)
            sage: b = a^2 + 5*a + 1
            sage: b.nth_root(5)
            19*a^4 + 2*a^3 + 2*a^2 + 16*a + 3
            sage: b.nth_root(7)
            Traceback (most recent call last):
            ...
            ValueError: no nth root
            sage: b.nth_root(4, all=True)
            []

        TESTS::

            sage: for p in [2,3,5,7,11]:  # long time, random because of PARI warnings
            ....:     for n in [2,5,10]:
            ....:         q = p^n
            ....:         K.<a> = GF(q)
            ....:         for r in (q-1).divisors():
            ....:             if r == 1: continue
            ....:             x = K.random_element()
            ....:             y = x^r
            ....:             assert y.nth_root(r)^r == y
            ....:             assert (y^41).nth_root(41*r)^(41*r) == y^41
            ....:             assert (y^307).nth_root(307*r)^(307*r) == y^307
            sage: k.<a> = GF(4)
            sage: a.nth_root(0,all=True)
            []
            sage: k(1).nth_root(0,all=True)
            [a, a + 1, 1]

        ALGORITHM:

        The default is currently an algorithm described in [Joh1999]_.

        AUTHOR:

        - David Roe (2010-02-13)
        """
        if self.is_zero():
            if n <= 0:
                if all: return []
                else: raise ValueError
            if all: return [self]
            else: return self
        if n < 0:
            self = ~self
            n = -n
        elif n == 0:
            if self == 1:
                if all: return [a for a in self.parent().list() if a != 0]
                else: return self
            else:
                if all: return []
                else: raise ValueError
        if extend:
            raise NotImplementedError
        n = Integer(n)
        return self._nth_root_common(n, all, algorithm, cunningham)

    def pth_power(self, int k=1):
        """
        Return the `(p^k)`-th power of self, where `p` is the
        characteristic of the field.

        INPUT:

        - ``k`` -- integer (default: 1, must fit in C int type)

        Note that if `k` is negative, then this computes the appropriate root.

        EXAMPLES::

            sage: F.<a> = GF(29^2)
            sage: z = a^2 + 5*a + 1
            sage: z.pth_power()
            19*a + 20
            sage: z.pth_power(10)
            10*a + 28
            sage: z.pth_power(-10) == z
            True
            sage: F.<b> = GF(2^12)
            sage: y = b^3 + b + 1
            sage: y == (y.pth_power(-3))^(2^3)
            True
            sage: y.pth_power(2)
            b^7 + b^6 + b^5 + b^4 + b^3 + b
        """
        p = self.additive_order()
        n = self.parent().degree()
        return self**(p**(k % n))

    frobenius = pth_power

    def pth_root(self, int k=1):
        """
        Return the `(p^k)`-th root of self, where `p` is the characteristic
        of the field.

        INPUT:

        - ``k`` -- integer (default: 1, must fit in C int type)

        Note that if `k` is negative, then this computes the appropriate power.

        EXAMPLES::

            sage: F.<b> = GF(2^12)
            sage: y = b^3 + b + 1
            sage: y == (y.pth_root(3))^(2^3)
            True
            sage: y.pth_root(2)
            b^11 + b^10 + b^9 + b^7 + b^5 + b^4 + b^2 + b
        """
        return self.pth_power(-k)

    def conjugate(self):
        """
        This methods returns the result of the Frobenius morphism
        in the case where the field is a quadratic extension, say
        `GF(q^2)`, where `q=p^k` is a prime power and `p` the
        characteristic of the field.

        OUTPUT:

        Instance of this class representing the image under
        the Frobenius morphisms.

        EXAMPLES::

            sage: F.<a> = GF(16)
            sage: b = a.conjugate(); b
            a + 1
            sage: a == b.conjugate()
            True

            sage: F.<a> = GF(27)
            sage: a.conjugate()
            Traceback (most recent call last):
            ...
            TypeError: cardinality of the field must be a square number

        TESTS:

        Check that :issue:`26761` is fixed::

            sage: # needs sage.libs.gap
            sage: G32 = GU(3,2)
            sage: g1, g2 = G32.gens()
            sage: m1 = g1.matrix()
            sage: m1.is_unitary()
            True
            sage: G32(m1) == g1
            True
        """
        k2 = self.parent().degree()
        if k2 % 2:
            raise TypeError("cardinality of the field must be a square number")
        k = k2 / 2

        return self.pth_power(k=k)

    def to_integer(self, reverse=False):
        r"""
        Return an integer representation of this finite field element
        obtained by lifting its representative polynomial to `\ZZ` and
        evaluating it at the characteristic `p`.

        If ``reverse`` is set to ``True`` (default: ``False``),
        the list of coefficients is reversed prior to evaluation.

        Inverse of :meth:`sage.rings.finite_rings.finite_field_base.FiniteField.from_integer`.

        EXAMPLES::

            sage: F.<t> = GF(7^5)
            sage: F(5).to_integer()
            5
            sage: t.to_integer()
            7
            sage: (t^2).to_integer()
            49
            sage: (t^2+1).to_integer()
            50
            sage: (t^2+t+1).to_integer()
            57

        ::

            sage: F.<t> = GF(2^8)
            sage: u = F.from_integer(0xd1)
            sage: bin(u.to_integer(False))
            '0b11010001'
            sage: bin(u.to_integer(True))
            '0b10001011'

        TESTS::

            sage: # needs sage.modules
            sage: p = random_prime(2^99)
            sage: k = randrange(2,10)
            sage: F.<t> = GF((p, k))
            sage: rev = bool(randrange(2))
            sage: u = F.random_element()
            sage: 0 <= u.to_integer(rev) < F.cardinality()
            True
            sage: F.from_integer(u.to_integer(rev), rev) == u
            True
            sage: n = randrange(F.cardinality())
            sage: F.from_integer(n, rev).to_integer(rev) == n
            True
        """
        if not reverse:
            try:
                return self._integer_representation()
            except AttributeError:
                pass
        p = self.parent().characteristic()
        f = self.polynomial().change_ring(ZZ)
        if reverse:
            f = f.reverse(self.parent().degree() - 1)
        return f(p)

    def to_bytes(self, byteorder='big'):
        r"""
        Return an array of bytes representing an integer.

        Internally relies on the python ``int.to_bytes()`` method.
        Length of byte array is determined from the field's order.

        INPUT:

        - ``byteorder`` -- string (default: ``'big'``); determines the byte order of
          the output; can only be ``'big'`` or ``'little'``

        EXAMPLES::

            sage: F.<z5> = GF(3^5)
            sage: a = z5^4 + 2*z5^3 + 1
            sage: a.to_bytes()
            b'\x88'

        ::

            sage: F.<z3> = GF(163^3)
            sage: a = 136*z3^2 + 10*z3 + 125
            sage: a.to_bytes()
            b'7)\xa3'
        """
        length = (self.parent().order().nbits() + 7) // 8
        return self.to_integer().to_bytes(length=length, byteorder=byteorder)

cdef class Cache_base(SageObject):
    cpdef FinitePolyExtElement fetch_int(self, number):
        r"""
        Given an integer less than `p^n` with base `2`
        representation `a_0 + a_1 \cdot 2 + \cdots + a_k 2^k`, this returns
        `a_0 + a_1 x + \cdots + a_k x^k`, where `x` is the
        generator of this finite field.

        EXAMPLES::

            sage: k.<a> = GF(2^48)
            sage: k._cache.fetch_int(2^33 + 2 + 1)                                      # needs sage.libs.ntl
            a^33 + a + 1
        """
        raise NotImplementedError("this must be implemented by subclasses")



# sage_setup: distribution = sagemath-categories
"""
Big O for various types (power series, `p`-adics, etc.)

.. SEEALSO::

    - `asymptotic expansions <../../../asymptotic/index.html>`_
    - `p-adic numbers <../../../padics/index.html>`_
    - `power series <../../../power_series/index.html>`_
    - `polynomials <../../../polynomial_rings/index.html>`_
"""

from sage.arith.misc import factor
from sage.misc.lazy_import import lazy_import

lazy_import('sage.rings.padics.factory', ['Qp', 'Zp'])
lazy_import('sage.rings.padics.padic_generic_element', 'pAdicGenericElement')
from sage.rings.polynomial.polynomial_element import Polynomial

try:
    from .laurent_series_ring_element import LaurentSeries
except ImportError:
    LaurentSeries = ()

try:
    from .puiseux_series_ring_element import PuiseuxSeries
except ImportError:
    PuiseuxSeries = ()

from sage.rings import (
    integer,
    multi_power_series_ring_element,
    power_series_ring_element,
    rational,
)


def O(*x, **kwds):
    """
    Big O constructor for various types.

    EXAMPLES:

    This is useful for writing power series elements::

        sage: R.<t> = ZZ[['t']]
        sage: (1+t)^10 + O(t^5)
        1 + 10*t + 45*t^2 + 120*t^3 + 210*t^4 + O(t^5)

    A power series ring is created implicitly if a polynomial
    element is passed::

        sage: R.<x> = QQ['x']
        sage: O(x^100)
        O(x^100)
        sage: 1/(1+x+O(x^5))
        1 - x + x^2 - x^3 + x^4 + O(x^5)
        sage: R.<u,v> = QQ[[]]
        sage: 1 + u + v^2 + O(u, v)^5
        1 + u + v^2 + O(u, v)^5

    This is also useful to create `p`-adic numbers::

        sage: O(7^6)                                                                    # needs sage.rings.padics
        O(7^6)
        sage: 1/3 + O(7^6)                                                              # needs sage.rings.padics
        5 + 4*7 + 4*7^2 + 4*7^3 + 4*7^4 + 4*7^5 + O(7^6)

    It behaves well with respect to adding negative powers of `p`::

        sage: a = O(11^-32); a                                                          # needs sage.rings.padics
        O(11^-32)
        sage: a.parent()                                                                # needs sage.rings.padics
        11-adic Field with capped relative precision 20

    There are problems if you add a rational with very negative
    valuation to an `O`-Term::

        sage: 11^-12 + O(11^15)                                                         # needs sage.rings.padics
        11^-12 + O(11^8)

    The reason that this fails is that the constructor doesn't know
    the right precision cap to use. If you cast explicitly or use
    other means of element creation, you can get around this issue::

        sage: # needs sage.rings.padics
        sage: K = Qp(11, 30)
        sage: K(11^-12) + O(11^15)
        11^-12 + O(11^15)
        sage: 11^-12 + K(O(11^15))
        11^-12 + O(11^15)
        sage: K(11^-12, absprec=15)
        11^-12 + O(11^15)
        sage: K(11^-12, 15)
        11^-12 + O(11^15)

    We can also work with `asymptotic expansions`_::

        sage: A.<n> = AsymptoticRing(growth_group='QQ^n * n^QQ * log(n)^QQ',            # needs sage.symbolic
        ....:                        coefficient_ring=QQ); A
        Asymptotic Ring <QQ^n * n^QQ * log(n)^QQ * Signs^n> over Rational Field
        sage: O(n)                                                                      # needs sage.symbolic
        O(n)

    Application with Puiseux series::

        sage: P.<y> = PuiseuxSeriesRing(ZZ)
        sage: y^(1/5) + O(y^(1/3))
        y^(1/5) + O(y^(1/3))
        sage: y^(1/3) + O(y^(1/5))
        O(y^(1/5))


    TESTS::

        sage: var('x, y')                                                               # needs sage.symbolic
        (x, y)
        sage: O(x)                                                                      # needs sage.symbolic
        Traceback (most recent call last):
        ...
        ArithmeticError: O(x) not defined
        sage: O(y)                                                                      # needs sage.symbolic
        Traceback (most recent call last):
        ...
        ArithmeticError: O(y) not defined
        sage: O(x, y)
        Traceback (most recent call last):
        ...
        ArithmeticError: O(x, y) not defined
        sage: O(4, 2)
        Traceback (most recent call last):
        ...
        ArithmeticError: O(4, 2) not defined
    """
    if len(x) > 1:
        if isinstance(x[0], multi_power_series_ring_element.MPowerSeries):
            return multi_power_series_ring_element.MO(x, **kwds)
        else:
            raise ArithmeticError("O(%s) not defined" %
                                  (', '.join(str(e) for e in x),))

    x = x[0]

    if isinstance(x, power_series_ring_element.PowerSeries):
        return x.parent()(0, x.degree(), **kwds)

    elif isinstance(x, Polynomial):
        if x.parent().ngens() != 1:
            raise NotImplementedError("completion only currently defined "
                                      "for univariate polynomials")
        if not x.is_monomial():
            raise NotImplementedError("completion only currently defined "
                                      "for the maximal ideal (x)")
        return x.parent().completion(x.parent().gen())(0, x.degree(), **kwds)

    elif isinstance(x, LaurentSeries):
        return LaurentSeries(x.parent(), 0).add_bigoh(x.valuation(), **kwds)

    elif isinstance(x, PuiseuxSeries):
        return x.add_bigoh(x.valuation(), **kwds)

    elif isinstance(x, (int, integer.Integer, rational.Rational)):
        # p-adic number
        if x <= 0:
            raise ArithmeticError("x must be a prime power >= 2")
        F = factor(x)
        if len(F) != 1:
            raise ArithmeticError("x must be prime power")
        p, r = F[0]
        if r >= 0:
            return Zp(p, prec=max(r, 20),
                      type='capped-rel')(0, absprec=r, **kwds)
        else:
            return Qp(p, prec=max(r, 20),
                      type='capped-rel')(0, absprec=r, **kwds)

    elif isinstance(x, pAdicGenericElement):
        return x.parent()(0, absprec=x.valuation(), **kwds)
    elif hasattr(x, 'O'):
        return x.O(**kwds)
    raise ArithmeticError("O(%s) not defined" % (x,))



# sage_setup: distribution = sagemath-categories
##############################################################################
#       Copyright (C) 2010 Nathann Cohen <nathann.cohen@gmail.com>
#  Distributed under the terms of the GNU General Public License (GPL)
#  The full text of the GPL is available at:
#                  http://www.gnu.org/licenses/
##############################################################################

from sage.structure.sage_object cimport SageObject

# We inherit from SageObject to make some testing infrastructure available.

cdef class GenericBackend (SageObject):
    cpdef int add_variable(self, lower_bound=*, upper_bound=*, binary=*, continuous=*, integer=*, obj=*, name=*) except -1
    cpdef int add_variables(self, int, lower_bound=*, upper_bound=*, binary=*, continuous=*, integer=*, obj=*, names=*) except -1
    cpdef set_variable_type(self, int variable, int vtype)
    cpdef set_sense(self, int sense)
    cpdef objective_coefficient(self, int variable, coeff=*)
    cpdef objective_constant_term(self, d=*)
    cpdef set_objective(self, list coeff, d=*)
    cpdef set_verbosity(self, int level)
    cpdef add_linear_constraint(self, coefficients, lower_bound, upper_bound, name=*)
    cpdef add_linear_constraint_vector(self, degree, coefficients, lower_bound, upper_bound, name=*)
    cpdef remove_constraint(self, int)
    cpdef remove_constraints(self, constraints)
    cpdef add_col(self, indices, coeffs)
    cpdef add_linear_constraints(self, int number, lower_bound, upper_bound, names=*)
    cpdef int solve(self) except -1
    cpdef get_objective_value(self)
    cpdef best_known_objective_bound(self)
    cpdef get_relative_objective_gap(self)
    cpdef get_variable_value(self, int variable)
    cpdef bint is_maximization(self) noexcept
    cpdef write_lp(self, name)
    cpdef write_mps(self, name, int modern)
    cpdef row(self, int i)
    cpdef int ncols(self) noexcept
    cpdef int nrows(self) noexcept
    cpdef bint is_variable_binary(self, int) noexcept
    cpdef bint is_variable_integer(self, int) noexcept
    cpdef bint is_variable_continuous(self, int) noexcept
    cpdef problem_name(self, name=*)
    cpdef row_bounds(self, int index)
    cpdef col_bounds(self, int index)
    cpdef row_name(self, int index)
    cpdef col_name(self, int index)
    cpdef variable_upper_bound(self, int index, value=*)
    cpdef variable_lower_bound(self, int index, value=*)
    cpdef solver_parameter(self, name, value=*)
    cpdef zero(self)
    cpdef base_ring(self)
    cpdef __copy__(self)
    cpdef copy(self)
    cpdef bint is_variable_basic(self, int index) noexcept
    cpdef bint is_variable_nonbasic_at_lower_bound(self, int index) noexcept
    cpdef bint is_slack_variable_basic(self, int index) noexcept
    cpdef bint is_slack_variable_nonbasic_at_lower_bound(self, int index) noexcept

    cdef object obj_constant_term

cpdef GenericBackend get_solver(constraint_generation=?, solver=?, base_ring=?)



# sage_setup: distribution = sagemath-categories
"""
Finite prime fields

AUTHORS:

- William Stein: initial version

- Martin Albrecht (2008-01): refactoring

TESTS::

    sage: k = GF(3)
    sage: TestSuite(k).run()
"""

#*****************************************************************************
#       Copyright (C) 2006 William Stein <wstein@gmail.com>
#       Copyright (C) 2008 Martin Albrecht <malb@informatik.uni-bremen.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************`

from sage.rings.finite_rings.finite_field_base import FiniteField as FiniteField_generic
from sage.categories.finite_fields import FiniteFields
_FiniteFields = FiniteFields()

import sage.rings.finite_rings.integer_mod_ring as integer_mod_ring
from sage.rings.integer import Integer
import sage.rings.finite_rings.integer_mod as integer_mod

from sage.rings.integer_ring import ZZ
from sage.rings.finite_rings.integer_mod_ring import IntegerModRing_generic

from sage.misc.persist import register_unpickle_override


class FiniteField_prime_modn(FiniteField_generic, integer_mod_ring.IntegerModRing_generic):
    r"""
    Finite field of order `p` where `p` is prime.

    EXAMPLES::

        sage: FiniteField(3)
        Finite Field of size 3

        sage: FiniteField(next_prime(1000))                                             # needs sage.rings.finite_rings
        Finite Field of size 1009
    """
    def __init__(self, p, check=True, modulus=None):
        """
        Return a new finite field of order `p` where `p` is prime.

        INPUT:

        - ``p`` -- integer at least 2

        - ``check`` -- boolean (default: ``True``); if ``False``, do not
          check ``p`` for primality

        EXAMPLES::

            sage: F = FiniteField(3); F
            Finite Field of size 3
        """
        p = Integer(p)
        if check and not p.is_prime():
            raise ArithmeticError("p must be prime")
        self.__char = p
        # FiniteField_generic does nothing more than IntegerModRing_generic, and
        # it saves a non trivial overhead
        integer_mod_ring.IntegerModRing_generic.__init__(self, p, category=_FiniteFields)

        # If modulus is None, it will be created on demand as x-1
        # by the modulus() method.
        if modulus is not None:
            self._modulus = modulus

    def __reduce__(self):
        """
        For pickling.

        EXAMPLES::

            sage: k = FiniteField(5); type(k)
            <class 'sage.rings.finite_rings.finite_field_prime_modn.FiniteField_prime_modn_with_category'>
            sage: k is loads(dumps(k))
            True
        """
        return self._factory_data[0].reduce_data(self)

    def _coerce_map_from_(self, S):
        """
        This is called implicitly by arithmetic methods.

        EXAMPLES::

            sage: k = GF(7)
            sage: e = k(6)
            sage: e * 2 # indirect doctest
            5
            sage: 12 % 7
            5

            sage: ZZ.residue_field(7).hom(GF(7))(1)  # See trac 11319
            1

            sage: # needs sage.rings.finite_rings sage.rings.number_field
            sage: K.<w> = QuadraticField(337)  # See trac 11319
            sage: pp = K.ideal(13).factor()[0][0]
            sage: RF13 = K.residue_field(pp)
            sage: RF13.hom([GF(13)(1)])
            Ring morphism:
             From: Residue field of Fractional ideal (-w - 18)
             To:   Finite Field of size 13
             Defn: 1 |--> 1

        Check that :issue:`19573` is resolved::

            sage: Integers(9).hom(GF(3))
            Natural morphism:
              From: Ring of integers modulo 9
              To:   Finite Field of size 3

            sage: Integers(9).hom(GF(5))
            Traceback (most recent call last):
            ...
            TypeError: natural coercion morphism from Ring of integers modulo 9 to Finite Field of size 5 not defined

        There is no coercion from a `p`-adic ring to its residue field::

            sage: GF(3).has_coerce_map_from(Zp(3))                                      # needs sage.rings.padics
            False
        """
        if S is int:
            return integer_mod.Int_to_IntegerMod(self)
        elif S is ZZ:
            return integer_mod.Integer_to_IntegerMod(self)
        elif isinstance(S, IntegerModRing_generic):
            from .residue_field import ResidueField_generic
            if (S.characteristic() % self.characteristic() == 0 and
                    (not isinstance(S, ResidueField_generic) or
                     S.degree() == 1)):
                try:
                    return integer_mod.IntegerMod_to_IntegerMod(S, self)
                except TypeError:
                    pass
        to_ZZ = ZZ._internal_coerce_map_from(S)
        if to_ZZ is not None:
            return integer_mod.Integer_to_IntegerMod(self) * to_ZZ

    def _convert_map_from_(self, R):
        """
        Conversion from `p`-adic fields.

        EXAMPLES::

            sage: # needs sage.rings.padics
            sage: GF(3).convert_map_from(Qp(3))
            Reduction morphism:
              From: 3-adic Field with capped relative precision 20
              To:   Finite Field of size 3
            sage: GF(3).convert_map_from(Zp(3))
            Reduction morphism:
              From: 3-adic Ring with capped relative precision 20
              To:   Finite Field of size 3
        """
        from sage.rings.padics.padic_generic import pAdicGeneric, ResidueReductionMap
        if isinstance(R, pAdicGeneric) and R.residue_field() is self:
            return ResidueReductionMap._create_(R, self)

    def construction(self):
        """
        Return the construction of this finite field (for use by
        ``sage.categories.pushout``).

        EXAMPLES::

            sage: GF(3).construction()
            (QuotientFunctor, Integer Ring)
        """
        return integer_mod_ring.IntegerModRing_generic.construction(self)

    def characteristic(self):
        r"""
        Return the characteristic of \code{self}.

        EXAMPLES::

            sage: k = GF(7)
            sage: k.characteristic()
            7
        """
        return self.__char

    def is_prime_field(self):
        """
        Return ``True`` since this is a prime field.

        EXAMPLES::

            sage: k.<a> = GF(3)
            sage: k.is_prime_field()
            True

            sage: # needs sage.rings.finite_rings
            sage: k.<a> = GF(3^2)
            sage: k.is_prime_field()
            False
        """
        return True

    def polynomial(self, name=None):
        """
        Return the polynomial ``name``.

        EXAMPLES::

            sage: k.<a> = GF(3)
            sage: k.polynomial()
            x
        """
        if name is None:
            name = self.variable_name()
        try:
            return self.__polynomial[name]
        except AttributeError:
            f = self[name]([0, 1])
            try:
                self.__polynomial[name] = f
            except (KeyError, AttributeError):
                self.__polynomial = {}
                self.__polynomial[name] = f
            return f

    def order(self):
        """
        Return the order of this finite field.

        EXAMPLES::

            sage: k = GF(5)
            sage: k.order()
            5
        """
        return self.__char

    def gen(self, n=0):
        """
        Return a generator of ``self`` over its prime field, which is a
        root of ``self.modulus()``.

        Unless a custom modulus was given when constructing this prime
        field, this returns `1`.

        INPUT:

        - ``n`` -- must be 0

        OUTPUT:

        An element `a` of ``self`` such that ``self.modulus()(a) == 0``.

        .. WARNING::

            This generator is not guaranteed to be a generator for the
            multiplicative group.  To obtain the latter, use
            :meth:`~sage.rings.finite_rings.finite_field_base.FiniteFields.multiplicative_generator()`
            or use the ``modulus="primitive"`` option when constructing
            the field.

        EXAMPLES::

            sage: k = GF(13)
            sage: k.gen()
            1

            sage: # needs sage.rings.finite_rings
            sage: k = GF(1009, modulus='primitive')
            sage: k.gen()  # this gives a primitive element
            11
            sage: k.gen(1)
            Traceback (most recent call last):
            ...
            IndexError: only one generator
        """
        if n:
            raise IndexError("only one generator")
        try:
            return self.__gen
        except AttributeError:
            pass

        try:
            self.__gen = -(self._modulus[0])
        except AttributeError:
            self.__gen = self.one()
        return self.__gen

    def __iter__(self):
        """
        Return an iterator over ``self``.

        EXAMPLES::

            sage: list(GF(7))
            [0, 1, 2, 3, 4, 5, 6]

        We can even start iterating over something that would be too big
        to actually enumerate::

            sage: # needs sage.rings.finite_rings
            sage: K = GF(next_prime(2^256))
            sage: all = iter(K)
            sage: next(all)
            0
            sage: next(all)
            1
            sage: next(all)
            2
        """
        yield self(0)
        i = one = self(1)
        while i:
            yield i
            i += one

    def degree(self):
        """
        Return the degree of ``self`` over its prime field.

        This always returns 1.

        EXAMPLES::

            sage: FiniteField(3).degree()
            1
        """
        return Integer(1)


register_unpickle_override(
    'sage.rings.finite_field_prime_modn', 'FiniteField_prime_modn',
    FiniteField_prime_modn)
